@@ -13,7 +13,7 @@ end
 trainlab=2*round(rand(1,1000))-1;
 %testdat=rand(100,1000);
 testlab=2*round(rand(1,1000))-1;
-sg('set_features', 'TRAIN', traindat);
+sg('set_features', 'TRAIN', traindat, 'DNA');
 sg('set_labels', 'TRAIN', trainlab);
 sg('send_command', 'set_kernel LINEAR CHAR 100');
 sg('send_command', 'init_kernel TRAIN');
@@ -21,13 +21,13 @@ sg('send_command', 'new_svm LIGHT');
 sg('send_command', 'c 5');
 sg('send_command', 'svm_train');
 [b, alphas]=sg('get_svm');
-sg('set_features', 'TEST', testdat);
+sg('set_features', 'TEST', testdat, 'DNA');
 sg('set_labels', 'TEST', testlab);
 sg('send_command', 'init_kernel TEST');
 out=sg('svm_classify');
 valerr=mean(testlab~=sign(out));
 
-sg('set_features', 'TRAIN', traindat);
+sg('set_features', 'TRAIN', traindat, 'DNA');
 sg('set_labels', 'TRAIN', trainlab);
 sg('send_command', 'set_kernel LINEAR CHAR 100');
 sg('send_command', 'init_kernel TRAIN');
@@ -35,7 +35,7 @@ sg('send_command', 'new_svm LIGHT');
 sg('send_command', 'c 5');
 sg('send_command', 'svm_train');
 [b2, alphas2]=sg('get_svm');
-sg('set_features', 'TEST', testdat);
+sg('set_features', 'TEST', testdat, 'DNA');
 sg('set_labels', 'TEST', testlab);
 sg('send_command', 'init_kernel_optimization');
 sg('send_command', 'init_kernel TEST');
