@@ -10,12 +10,13 @@
 
 #include "classifier/Classifier.h"
 
-CClassifier::CClassifier() : labels(NULL)
+CClassifier::CClassifier() : CSGObject(), max_train_time(0), labels(NULL)
 {
 }
 
 CClassifier::~CClassifier()
 {
+    SG_UNREF(labels);
 }
 
 CLabels* CClassifier::classify(CLabels* output)
