@@ -30,12 +30,18 @@ class CClassifier : public CSGObject
 		virtual bool load(FILE* srcfile) { ASSERT(srcfile); return false; }
 		virtual bool save(FILE* dstfile) { ASSERT(dstfile); return false; }
 
-		virtual inline void set_labels(CLabels* lab) { labels=lab; }
-		virtual inline CLabels* get_labels() { return labels; }
+		virtual inline void set_labels(CLabels* lab) { SG_REF(lab);; labels=lab; }
+		virtual inline CLabels* get_labels() { SG_REF(labels); return labels; }
+		virtual inline DREAL get_label(INT i) { return labels->get_label(i); }
 
-		virtual EClassifierType get_classifier_type() { return CT_NONE; }
+		inline void set_max_train_time(DREAL t) { max_train_time=t; }
+		inline DREAL get_max_train_time() { return max_train_time; }
+
+		virtual inline EClassifierType get_classifier_type() { return CT_NONE; }
 
 	protected:
+		DREAL max_train_time;
+
 		CLabels* labels;
 };
 #endif
