@@ -20,9 +20,10 @@ class CGPBTSVM : public CSVM
 {
 	public:
 		CGPBTSVM();
+		CGPBTSVM(DREAL C, CKernel* k, CLabels* lab);
 		virtual ~CGPBTSVM();
 		virtual bool train();
-		inline EClassifierType get_classifier_type() { return CT_GPBT; }
+		virtual inline EClassifierType get_classifier_type() { return CT_GPBT; }
 
 	protected:
 		struct svm_model* model;
