@@ -17,13 +17,13 @@
 
 #include <stdio.h>
 
-class CLibSVMOneclass : public CSVM
+class CLibSVMOneClass : public CSVM
 {
 	public:
-		CLibSVMOneclass();
-		virtual ~CLibSVMOneclass();
+		CLibSVMOneClass();
+		virtual ~CLibSVMOneClass();
 		virtual bool train();
-		inline EClassifierType get_classifier_type() { return CT_LIBSVMONECLASS; }
+		virtual inline EClassifierType get_classifier_type() { return CT_LIBSVMONECLASS; }
 
 	protected:
 		svm_problem problem;
