@@ -1,12 +1,3 @@
-/*
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * Written (W) 1999-2007 Soeren Sonnenburg
- */
-
 /******************************************************************************
  ***        GPDT - Gradient Projection Decomposition Technique              ***
  ******************************************************************************
@@ -72,9 +63,10 @@
  *** File:     gpdtsolve.cpp                                                ***
  *** Type:     scalar                                                       ***
  *** Version:  1.0                                                          ***
- *** Date:     October, 2005                                                ***
- *** Revision: 1                                                            ***
+ *** Date:     November, 2006                                                ***
+ *** Revision: 2                                                            ***
  ***                                                                        ***
+ *** SHOGUN adaptions  Written (W) 2006-2007 Soeren Sonnenburg              ***
  ******************************************************************************/
 #include <math.h>
 #include <stdio.h>
@@ -85,6 +77,7 @@
 #include "classifier/svm/gpm.h"
 #include "classifier/svm/gpdt.h"
 #include "classifier/svm/gpdtsolve.h"
+#include "lib/Signal.h"
 #include "lib/io.h"
 
 #define y_in(i)      y[index_in[(i)]]
@@ -746,8 +739,6 @@ int QPproblem::Preprocess1(sKernel* kernel, int *aux, int *sv)
 
   if (sl < 500)
       sp_D = (float *)malloc(sl*sl * sizeof(float));
-  else
-	  SG_ERROR("sl<500 expect me to die\n");
 
   for (i = 0; i < sl; i++)
        sp_h[i] = -1.0;
@@ -1164,15 +1155,15 @@ double QPproblem::gpdtsolve(double *solution)
       {
         if (ker_type == 2)
         {
-            SG_INFO( "\n WARNING: inner subproblem hard to solve;");
-            SG_INFO( " setting a smaller -q or");
-            SG_INFO( " tuning -c and -g options might help.\n");
+            SG_INFO("\n WARNING: inner subproblem hard to solve;");
+            SG_INFO(" setting a smaller -q or");
+            SG_INFO(" tuning -c and -g options might help.\n");
         }
         else
         {
-            SG_INFO( "\n WARNING: inner subproblem hard to solve;");
-            SG_INFO( " set a smaller -q or");
-            SG_INFO( " try a better data scaling.\n");
+            SG_INFO("\n WARNING: inner subproblem hard to solve;");
+            SG_INFO(" set a smaller -q or");
+            SG_INFO(" try a better data scaling.\n");
         }
       }
 
@@ -1239,7 +1230,27 @@ double QPproblem::gpdtsolve(double *solution)
         }
     }
 
-    if (ker_type != 0)  // nonlinear kernel
+	// in case of LINADD enabled use faster linadd variant
+	if (KER->get_kernel()->has_property(KP_LINADD) && get_linadd_enabled())
+	{
+		KER->get_kernel()->clear_normal() ;
+
+		for (j = 0; j < nzin; j++)
+			KER->get_kernel()->add_to_normal(indnzin[j], grad[j]);
+
+        if (nit == 0 && PreprocessMode > 0)
+		{
+			for (j = 0; j < nzout; j++)
+			{
+				jin = indnzout[j];
+				KER->get_kernel()->add_to_normal(jin, alpha[jin] * y[jin]);
+			}
+		}
+
+        for (i = 0; i < ell; i++)
+            st[i] += KER->get_kernel()->compute_optimized(i);
+	}
+	else  // nonlinear kernel
     {
         k = Cache->DivideMP(ing, indnzin, nzin);
         for (j = 0; j < k; j++)
@@ -1266,22 +1277,6 @@ double QPproblem::gpdtsolve(double *solution)
                  "  G*x0 time: %.2lf\n", (double)(clock()-ti2)/CLOCKS_PER_SEC);
         }
     }
-    else                // linear kernel
-    {
-        for (i = 0; i < dim; i++) weight[i] = 0.0;
-
-        for (j = 0; j < nzin; j++)
-            KER->Add(weight, indnzin[j], grad[j]);
-        if (nit == 0 && PreprocessMode > 0)
-            for (j = 0; j < nzout; j++)
-            {
-                jin = indnzout[j];
-                KER->Add(weight, jin, alpha[jin] * y[jin]);
-            }
-
-        for (i = 0; i < ell; i++)
-            st[i] += KER->Prod(weight, i);
-    }
 
     /*** sort the vectors for cache managing ***/
 
@@ -1310,7 +1305,7 @@ double QPproblem::gpdtsolve(double *solution)
     }
     Cache->Iteration();
     nit = nit+1;
-  } while (!optimal());
+  } while (!optimal() && !(CSignal::cancel_computations()));
   /* End of the problem resolution loop                                      */
   /***************************************************************************/
 
