@@ -41,7 +41,7 @@ extern "C" {
 
 void nrerror(CHAR error_text[])
 {
-	SG_SWARNING("terminating optimizer - %s\n", error_text);
+	SG_SDEBUG("terminating optimizer - %s\n", error_text);
  // exit(1); 
 }
 
@@ -57,6 +57,9 @@ void nrerror(CHAR error_text[])
 #ifdef HAVE_LAPACK
 bool choldc(double* a, int n, double* p)
 {
+	if (n<=0)
+		return false;
+
 	double* a2=new double[n*n];
 
 	for (int i=0; i<n; i++)
@@ -79,7 +82,7 @@ bool choldc(double* a, int n, double* p)
 	}
 
 	if (result>0)
-		SG_SWARNING("Choldc failed, matrix not positive definite\n");
+		SG_SDEBUG("Choldc failed, matrix not positive definite\n");
 
 	delete[] a2;
 	
