@@ -25,11 +25,10 @@
 
 #include "lib/io.h"
 #include "lib/Mathematics.h"
+#include "features/SparseFeatures.h"
 #include "classifier/svm/ssl.h"
 
 #define VERBOSE 1
-#define LOG2(x) 1.4426950408889634*log(x) 
-// for compatibility issues, not using log2
 
 void ssl_train(struct data *Data, 
 		struct options *Options, 
@@ -66,7 +65,7 @@ void ssl_train(struct data *Data,
 			optimality=DA_S3VM(Data,Options,Weights,Outputs);
 			break;
 		default:
-			;
+			SG_SERROR("Algorithm unspecified");
 	}
 	return;
 } 
@@ -81,9 +80,7 @@ int CGLS(const struct data *Data,
 	/* Disassemble the structures */
 	int active = Subset->d;
 	int *J = Subset->vec;
-	double *val = Data->val;
-	int *row = Data->rowptr;
-	int *col = Data->colind;
+	CSparseFeatures<DREAL>* features=Data->features;
 	double *Y = Data->Y;
 	double *C = Data->C;
 	int n  = Data->n;
@@ -106,8 +103,14 @@ int CGLS(const struct data *Data,
 	for(register int j=0; j < active; j++)
 	{
 		ii=J[j];
-		for(register int i=row[ii]; i < row[ii+1]; i++)
-			r[col[i]]+=val[i]*z[j];
+		INT num_entries=0;
+		bool free_vec=false;
+
+		TSparseEntry<DREAL>* vec=features->get_sparse_feature_vector(ii, num_entries, free_vec);
+		for (int i=0; i<num_entries; i++)
+			r[vec[i].feat_index]+= vec[i].entry*z[j];
+		features->free_sparse_feature_vector(vec, num_entries, free_vec);
+		r[n-1]+=Options->bias*z[j]; //bias (modelled as last dim)
 	}
 	double *p = new double[n];   
 	double omega1 = 0.0;
@@ -138,8 +141,14 @@ int CGLS(const struct data *Data,
 		{
 			ii=J[i];
 			t=0.0;
-			for(j=row[ii]; j < row[ii+1]; j++)
-				t+=val[j]*p[col[j]];
+			INT num_entries=0;
+			bool free_vec=false;
+			TSparseEntry<DREAL>* vec=features->get_sparse_feature_vector(ii,
+					num_entries, free_vec);
+			for (j=0; j<num_entries; j++)
+				t+=vec[j].entry*p[vec[j].feat_index];
+			features->free_sparse_feature_vector(vec, num_entries, free_vec);
+			t+=Options->bias*p[n-1]; //bias (modelled as last dim)
 			q[i]=t;
 			omega_q += C[ii]*t*t;
 		}       
@@ -162,8 +171,14 @@ int CGLS(const struct data *Data,
 		{
 			ii=J[j];
 			t=z[j];
-			for(i=row[ii]; i < row[ii+1]; i++)
-				r[col[i]]+=val[i]*t;
+			INT num_entries=0;
+			bool free_vec=false;
+
+			TSparseEntry<DREAL>* vec=features->get_sparse_feature_vector(ii, num_entries, free_vec);
+			for (i=0; i<num_entries; i++)
+				r[vec[i].feat_index]+= vec[i].entry*t;
+			features->free_sparse_feature_vector(vec, num_entries, free_vec);
+			r[n-1]+=Options->bias*t; //bias (modelled as last dim)
 		}
 		omega1 = 0.0;
 		for(i = n ; i-- ;)
@@ -171,7 +186,6 @@ int CGLS(const struct data *Data,
 			r[i] -= lambda*beta[i];
 			omega1 += r[i]*r[i];
 		}
-		SG_SDEBUG("...%d(%f)", cgiter,omega1);
 		if(omega1 < epsilon2*omega_z)
 		{
 			optimality=1;
@@ -201,9 +215,7 @@ int L2_SVM_MFN(const struct data *Data,
 		int ini)
 { 
 	/* Disassemble the structures */  
-	double *val = Data->val;
-	int *row = Data->rowptr;
-	int *col = Data->colind;
+	CSparseFeatures<DREAL>* features=Data->features;
 	double *Y = Data->Y;
 	double *C = Data->C;
 	int n  = Data->n;
@@ -273,8 +285,16 @@ int L2_SVM_MFN(const struct data *Data,
 		{
 			ii=ActiveSubset->vec[i];   
 			t=0.0;
-			for(register int j=row[ii]; j < row[ii+1]; j++)
-				t+=val[j]*w_bar[col[j]];
+
+			INT num_entries=0;
+			bool free_vec=false;
+
+			TSparseEntry<DREAL>* vec=features->get_sparse_feature_vector(ii, num_entries, free_vec);
+			for (int j=0; j<num_entries; j++)
+				t+=vec[j].entry*w_bar[vec[j].feat_index];
+			features->free_sparse_feature_vector(vec, num_entries, free_vec);
+			t+=Options->bias*w_bar[n-1]; //bias (modelled as last dim)
+
 			o_bar[ii]=t;
 		}
 		if(ini==0) {Options->cgitermax=CGITERMAX; ini=1;};
@@ -434,7 +454,6 @@ double line_search(double *w,
 	delete [] deltas;
 	return (-L/(R-L));
 } 
-
 int TSVM_MFN(const struct data *Data, 
 		struct options *Options, 
 		struct vector_double *Weights,
@@ -447,7 +466,7 @@ int TSVM_MFN(const struct data *Data,
 	SG_SDEBUG("Initializing weights, unknown labels");
 	GetLabeledData(Data_Labeled,Data); /* gets labeled data and sets C=1/l */
 	L2_SVM_MFN(Data_Labeled, Options, Weights,Outputs_Labeled,0);
-	Clear(Data_Labeled);
+	///FIXME Clear(Data_Labeled);
 	/* Use this weight vector to classify R*u unlabeled examples as
 	   positive*/   
 	int p=0,q=0; 
@@ -460,8 +479,15 @@ int TSVM_MFN(const struct data *Data,
 		if(Data->Y[i]==0.0)
 		{
 			t=0.0;
-			for(int j=Data->rowptr[i]; j < Data->rowptr[i+1]; j++)
-				t+=Data->val[j]*Weights->vec[Data->colind[j]];
+			INT num_entries=0;
+			bool free_vec=false;
+
+			TSparseEntry<DREAL>* vec=Data->features->get_sparse_feature_vector(i, num_entries, free_vec);
+			for (int j=0; j<num_entries; j++)
+				t+=vec[j].entry*Weights->vec[vec[j].feat_index];
+			Data->features->free_sparse_feature_vector(vec, num_entries, free_vec);
+			t+=Options->bias*Weights->vec[Data->n-1]; //bias (modelled as last dim)
+
 			Outputs->vec[i]=t;
 			Data->C[i]=lambda_0*1.0/Data->u;
 			JU[q]=i;
@@ -659,9 +685,7 @@ int optimize_w(const struct data *Data,
 		int ini)
 { 
 	int i,j;
-	double *val = Data->val;
-	int *row = Data->rowptr;
-	int *col = Data->colind;
+	CSparseFeatures<DREAL>* features=Data->features;
 	int n  = Data->n;
 	int m  = Data->m;
 	int u  = Data->u;
@@ -770,7 +794,6 @@ int optimize_w(const struct data *Data,
 	double delta=0.0;
 	double t=0.0;
 	int ii = 0;
-
 	while(iter<MFNITERMAX)
 	{
 		iter++;
@@ -785,8 +808,15 @@ int optimize_w(const struct data *Data,
 		{
 			ii=ActiveSubset->vec[i];   
 			t=0.0;
-			for(j=row[ii]; j < row[ii+1]; j++)
-				t+=val[j]*w_bar[col[j]];
+			INT num_entries=0;
+			bool free_vec=false;
+
+			TSparseEntry<DREAL>* vec=features->get_sparse_feature_vector(ii, num_entries, free_vec);
+			for (j=0; j<num_entries; j++)
+				t+=vec[j].entry*w_bar[vec[j].feat_index];
+			features->free_sparse_feature_vector(vec, num_entries, free_vec);
+			t+=Options->bias*w_bar[n-1]; //bias (modelled as last dim)
+
 			o_bar[ii]=t;
 		}
 		// make o_bar consistent in the bottom half      
@@ -1038,7 +1068,7 @@ double entropy(const double *p, int u)
 	{
 		q=p[i];
 		if(q>0 && q<1)
-			h+= -(q*LOG2(q) + (1-q)*LOG2(1-q));
+			h+= -(q*CMath::log2(q) + (1-q)*CMath::log2(1-q));
 	}
 	return h/u;
 }
@@ -1056,7 +1086,7 @@ double KL(const double *p, const double *q, int u)
 		if(p1<1-1e-8) p1+=1e-8;
 		if(q1>1-1e-8) q1-=1e-8;
 		if(q1<1-1e-8) q1+=1e-8;
-		g= (p1*LOG2(p1/q1) + (1-p1)*LOG2((1-p1)/(1-q1)));
+		g= (p1*CMath::log2(p1/q1) + (1-p1)*CMath::log2((1-p1)/(1-q1)));
 		if(CMath::abs(g)<1e-12 || isnan(g)) g=0.0;
 		h+=g;
 	}
@@ -1091,16 +1121,9 @@ void initialize(struct vector_int *A, int k)
 	A->d   = k;
 	return;
 }
-void Write(const char *file_name, 
-		const struct vector_double *somevector)
-{
-	FILE* fp = fopen(file_name,"w");
-	for(int i=0;i<somevector->d;i++)
-		fprintf(fp,"%g\n",somevector->vec[i]);
-	return;
-} 
 void GetLabeledData(struct data *D, const struct data *Data)
 {
+	/*FIXME
 	int *J = new int[Data->l];
 	D->C   = new double[Data->l];
 	D->Y   = new double[Data->l];
@@ -1138,35 +1161,5 @@ void GetLabeledData(struct data *D, const struct data *Data)
 	D->m=Data->l;
 	D->n=Data->n;
 	D->u=0;
-	delete [] J;
+	delete [] J;*/
 }
-void SetData(struct data *a, int m,int n, int l,int u, int nz, double *VAL, int *R, int *C, double *Y, double *COSTS)
-{
-	a->m=m;
-	a->u=u;
-	a->l=m-u;
-	a->n=n;
-	a->nz=nz;
-	a->val=VAL;
-	a->rowptr=R;
-	a->colind=C;
-	a->Y=Y;
-	a->C=COSTS;
-	return;
-}
-void Clear(struct data *a)
-{     
-	delete [] a->val;
-	delete [] a->rowptr;
-	delete [] a->colind;
-	delete [] a->Y;
-	delete [] a->C;
-	delete [] a;
-	return;
-}
-void Clear(struct vector_double *c)
-{ delete[] c->vec; delete [] c; return;}
-void Clear(struct vector_int *c)
-{ delete[] c->vec; delete [] c; return;}
-void Clear(struct options *opt)
-{ delete[] opt; delete [] opt; return;}
