@@ -19,10 +19,10 @@
 #include "features/Alphabet.h"
 #include "lib/common.h"
 #include "lib/io.h"
+#include "lib/DynamicArray.h"
 #include "lib/File.h"
 #include "lib/Mathematics.h"
 
-#include <math.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <dirent.h>
@@ -42,16 +42,37 @@ template <class T> struct T_STRING
 template <class ST> class CStringFeatures: public CFeatures
 {
 	public:
-	CStringFeatures(CAlphabet* alpha) : CFeatures(0), num_vectors(0), features(NULL), max_string_length(0), order(0), symbol_mask_table(NULL)
+	CStringFeatures(E_ALPHABET alpha) : CFeatures(0), num_vectors(0), features(NULL), 
+		single_string(NULL),length_of_single_string(0),
+		max_string_length(0), order(0), symbol_mask_table(NULL),
+		selected_vector(0)
 	{
 		alphabet=new CAlphabet(alpha);
-		ASSERT(alpha);
+		ASSERT(alphabet);
+		num_symbols=alphabet->get_num_symbols();
+		original_num_symbols=num_symbols;
+	}
+	CStringFeatures(CAlphabet* alpha) : CFeatures(0), num_vectors(0), features(NULL), 
+		single_string(NULL),length_of_single_string(0),
+		max_string_length(0), order(0), symbol_mask_table(NULL),
+		selected_vector(0)
+	{
+        ASSERT(alpha);
+		alphabet=new CAlphabet(alpha);
+		ASSERT(alphabet);
 		num_symbols=alphabet->get_num_symbols();
 		original_num_symbols=num_symbols;
 	}
 
-	CStringFeatures(const CStringFeatures & orig) : CFeatures(orig), num_vectors(orig.num_vectors), max_string_length(orig.max_string_length), num_symbols(orig.num_symbols), original_num_symbols(orig.original_num_symbols), order(orig.order)
+	CStringFeatures(const CStringFeatures & orig) : CFeatures(orig), num_vectors(orig.num_vectors),
+		single_string(orig.single_string),
+		length_of_single_string(orig.length_of_single_string),
+		max_string_length(orig.max_string_length),
+		num_symbols(orig.num_symbols), original_num_symbols(orig.original_num_symbols),
+		order(orig.order), selected_vector(orig.selected_vector)
 	{
+		ASSERT(orig.single_string == NULL); //not implemented
+
 		alphabet=new CAlphabet(orig.alphabet);
 
 		if (orig.features)
@@ -76,7 +97,10 @@ template <class ST> class CStringFeatures: public CFeatures
 		}
 	}
 
-	CStringFeatures(char* fname, E_ALPHABET alpha=DNA) : CFeatures(fname), num_vectors(0), features(NULL), max_string_length(0), order(0), symbol_mask_table(NULL)
+	CStringFeatures(char* fname, E_ALPHABET alpha=DNA) : CFeatures(fname), num_vectors(0), 
+		features(NULL), single_string(NULL), length_of_single_string(0),
+		max_string_length(0), order(0), symbol_mask_table(NULL),
+		selected_vector(0)
 	{
 		alphabet=new CAlphabet(alpha);
 		num_symbols=alphabet->get_num_symbols();
@@ -86,28 +110,39 @@ template <class ST> class CStringFeatures: public CFeatures
 
 	virtual ~CStringFeatures()
 	{
-		delete alphabet;
-		alphabet=NULL;
 		cleanup();
+
+		delete alphabet;
 	}
 
 	void cleanup()
 	{
-		for (int i=0; i<num_vectors; i++)
+		if (single_string)
 		{
-			delete[] features[i].string;
-			features[i].length=0;
+			delete[] single_string;
+			single_string=NULL;
 		}
+		else
+		{
+			for (int i=0; i<num_vectors; i++)
+			{
+				delete[] features[i].string;
+				features[i].length=0;
+			}
+		}
+		num_vectors=0;
 		delete[] features;
 
 		delete[] symbol_mask_table;
+		alphabet->clear_histogram();
 	}
 
-	inline virtual EFeatureClass get_feature_class() { return C_STRING ; } ;
+	inline virtual EFeatureClass get_feature_class() { return C_STRING; }
 	inline virtual EFeatureType get_feature_type();
 
 	inline CAlphabet* get_alphabet()
 	{
+		SG_REF(alphabet);
 		return alphabet;
 	}
 
@@ -149,9 +184,9 @@ template <class ST> class CStringFeatures: public CFeatures
 		return features[num].string;
 	}
 
-	/** get feature vector for sample num
+	/** set feature vector for sample num
 	  @param num index of feature vector
-	  @param len length is returned by reference
+	  @param len length of the string
 	  */
 	virtual void set_feature_vector(INT num, ST* string, INT len)
 	{
@@ -194,7 +229,7 @@ template <class ST> class CStringFeatures: public CFeatures
 	// order used for higher order mapping
 	inline INT get_order() { return order; }
 
-	// a higher order mapped symbol will be shaped such that the symbols in
+	// a higher order mapped symbol will be shaped such that the symbols
 	// specified by bits in the mask will be returned.
 	inline ST get_masked_symbols(ST symbol, BYTE mask)
 	{
@@ -202,6 +237,20 @@ template <class ST> class CStringFeatures: public CFeatures
 		return symbol_mask_table[mask] & symbol;
 	}
 
+	//shift offset to the left by amount
+	inline ST shift_offset(ST offset, INT amount)
+	{
+		ASSERT(alphabet);
+		return (offset << (amount*alphabet->get_num_bits()));
+	}
+
+	//shift symbol to the right by amount (taking care of custom symbol sizes)
+	inline ST shift_symbol(ST symbol, INT amount)
+	{
+		ASSERT(alphabet);
+		return (symbol >> (amount*alphabet->get_num_bits()));
+	}
+
 	virtual bool load(CHAR* fname)
 	{
 		SG_INFO( "loading...\n");
@@ -304,9 +353,6 @@ template <class ST> class CStringFeatures: public CFeatures
 						strings[num].length=filesize;
 						max_len=CMath::max(max_len, strings[num].length);
 
-						//compute histogram for char/byte
-						alphabet->add_string_to_histogram((BYTE*) strings[num].string, (LONG) strings[num].length);
-						
 						num++;
 						fclose(f);
 					}
@@ -327,20 +373,38 @@ template <class ST> class CStringFeatures: public CFeatures
 		return false;
 	}
 
-	void set_features(T_STRING<ST>* p_features, INT p_num_vectors, INT p_max_string_length)
+	bool set_features(T_STRING<ST>* p_features, INT p_num_vectors, INT p_max_string_length)
 	{
-		cleanup();
-		this->features=p_features;
-		this->num_vectors=p_num_vectors;
-		this->max_string_length=p_max_string_length;
-	}
+		if (p_features)
+		{
+			CAlphabet* alpha=new CAlphabet(alphabet);
+			ASSERT(alpha);
 
-	void copy_features(T_STRING<ST>* p_features, INT p_num_vectors, INT p_max_string_length)
-	{
-		cleanup();
-		this->features=p_features;
-		this->num_vectors=p_num_vectors;
-		this->max_string_length=p_max_string_length;
+			//compute histogram for char/byte
+			for (INT i=0; i<p_num_vectors; i++)
+				alpha->add_string_to_histogram( p_features[i].string, p_features[i].length);
+
+			SG_INFO("max_value_in_histogram:%d\n", alpha->get_max_value_in_histogram());
+			SG_INFO("num_symbols_in_histogram:%d\n", alpha->get_num_symbols_in_histogram());
+
+			if (alpha->check_alphabet_size() && alpha->check_alphabet())
+			{
+				cleanup();
+
+				delete alphabet;
+				alphabet=alpha;
+
+				this->features=p_features;
+				this->num_vectors=p_num_vectors;
+				this->max_string_length=p_max_string_length;
+
+				return true;
+			}
+			else
+				delete alpha;
+		}
+
+		return false;
 	}
 
 	virtual bool save(CHAR* dest)
@@ -357,25 +421,125 @@ template <class ST> class CStringFeatures: public CFeatures
 
 		for (INT i=0; i<get_num_preproc(); i++)
 		{ 
-			SG_INFO( "preprocessing using preproc %s\n", get_preproc(i)->get_name());
-			bool ok=((CStringPreProc<ST>*) get_preproc(i))->apply_to_string_features(this) ;
+			if ( (!is_preprocessed(i) || force_preprocessing) )
+			{
+				set_preprocessed(i);
 
-			if (!ok)
-				return false;
+				SG_INFO( "preprocessing using preproc %s\n", get_preproc(i)->get_name());
+				if (((CStringPreProc<ST>*) get_preproc(i))->apply_to_string_features(this))
+					return false;
+			}
 		}
 		return true;
 	}
-
-	inline bool obtain_from_char(CStringFeatures<CHAR>* sf, INT start, INT p_order, INT gap)
+	
+	///slides a window of size window_size over the current single string
+	///step_size is the amount by which the window is shifted.
+	///creates (string_len-window_size)/step_size many feature obj
+	///if skip is nonzero, skip the first 'skip' characters of each string
+	INT obtain_by_sliding_window(INT window_size, INT step_size, INT skip=0)
 	{
-		return obtain_from_char_features(sf, start, p_order, gap);
+		ASSERT(step_size>0);
+		ASSERT(window_size>0);
+		ASSERT(num_vectors==1 || single_string);
+		ASSERT(max_string_length>=window_size ||
+				( single_string && length_of_single_string>=window_size));
+
+		//in case we are dealing with a single remapped string
+		//allow remapping
+		if (single_string)
+			num_vectors= (length_of_single_string-window_size)/step_size + 1;
+		else if (num_vectors==1)
+		{
+			num_vectors= (max_string_length-window_size)/step_size + 1;
+			length_of_single_string=max_string_length;
+		}
+		
+		T_STRING<ST>* f=new T_STRING<ST>[num_vectors];
+		INT offs=0;
+		for (INT i=0; i<num_vectors; i++)
+		{
+			f[i].string=&features[0].string[offs+skip];
+			f[i].length=window_size-skip;
+			offs+=step_size;
+		}
+		single_string=features[0].string;
+		delete[] features;
+		features=f;
+		selected_vector=0;
+		max_string_length=window_size-skip;
+
+		return num_vectors;
+	}
+
+	///extracts windows of size window_size from first string
+	///using the positions in list
+	INT obtain_by_position_list(INT window_size, CDynamicArray<INT>* positions, INT skip=0)
+	{
+		ASSERT(positions);
+		ASSERT(window_size>0);
+		ASSERT(num_vectors==1 || single_string);
+		ASSERT(max_string_length>=window_size ||
+				( single_string && length_of_single_string>=window_size));
+
+		num_vectors= positions->get_num_elements();
+		ASSERT(num_vectors>0);
+
+		INT len;
+
+		//in case we are dealing with a single remapped string
+		//allow remapping
+		if (single_string)
+			len=length_of_single_string;
+		else
+		{
+			single_string=features[0].string;
+			len=max_string_length;
+			length_of_single_string=max_string_length;
+		}
+		
+		T_STRING<ST>* f=new T_STRING<ST>[num_vectors];
+		ASSERT(f);
+
+		for (INT i=0; i<num_vectors; i++)
+		{
+			INT p=positions->get_element(i);
+
+			if (p>=0 && p<=len-window_size)
+			{
+				f[i].string=&features[0].string[p+skip];
+				f[i].length=window_size-skip;
+			}
+			else
+			{
+				num_vectors=1;
+				max_string_length=len;
+				features[0].length=len;
+				single_string=NULL;
+				delete[] f;
+				SG_ERROR("window (size:%d) starting at position[%d]=%d does not fit in sequence(len:%d)\n",
+						window_size, i, p, len);
+				return -1;
+			}
+		}
+		delete[] features;
+		features=f;
+		selected_vector=0;
+		max_string_length=window_size-skip;
+
+		return num_vectors;
+	}
+
+	inline bool obtain_from_char(CStringFeatures<CHAR>* sf, INT start, INT p_order, INT gap, bool rev)
+	{
+		return obtain_from_char_features(sf, start, p_order, gap, rev);
 	}
 
     template <class CT>
-	bool obtain_from_char_features(CStringFeatures<CT>* sf, INT start, INT p_order, INT gap)
+	bool obtain_from_char_features(CStringFeatures<CT>* sf, INT start, INT p_order, INT gap, bool rev)
 	{
 		ASSERT(sf);
-		this->order=order;
+		this->order=p_order;
 		cleanup();
 		delete[] symbol_mask_table;
 		symbol_mask_table=new ST[256];
@@ -387,14 +551,6 @@ template <class ST> class CStringFeatures: public CFeatures
 		ASSERT(features);
 
 		CAlphabet* alpha=sf->get_alphabet();
-		//alpha->clear_histogram();
-		//for (INT i=0; i<num_vectors; i++)
-		//{
-		//	INT len=-1;
-		//	CT* c=sf->get_feature_vector(i, len);
-		//	alpha->add_string_to_histogram(c, len);
-		//}
-
 		ASSERT(alpha->get_num_symbols_in_histogram() > 0);
 
 		SG_DEBUG( "%1.0llf symbols in StringFeatures<*>\n", sf->get_num_symbols());
@@ -434,7 +590,13 @@ template <class ST> class CStringFeatures: public CFeatures
 		{
 			INT len=0;
 			ST* fv=get_feature_vector(line, len);
-			translate_from_single_order(fv, len, start+gap, p_order+gap, max_val, gap);
+
+			if (rev)
+				translate_from_single_order_reversed(fv, len, start+gap, p_order+gap, max_val, gap);
+			else
+				translate_from_single_order(fv, len, start+gap, p_order+gap, max_val, gap);
+			//translate_from_single_order(fv, len, start, p_order, max_val);
+			//translate_from_single_order_reversed(fv, len, start, p_order, max_val);
 
 			/* fix the length of the string -- hacky */
 			features[line].length-=start+gap ;
@@ -449,6 +611,7 @@ template <class ST> class CStringFeatures: public CFeatures
 		for (INT i=0; i<256; i++)
 		{
 			BYTE bits=(BYTE) i;
+			symbol_mask_table[i]=0;
 
 			for (INT j=0; j<8; j++)
 			{
@@ -469,10 +632,10 @@ template <class ST> class CStringFeatures: public CFeatures
 		INT i,j;
 		ST value=0;
 
-		for (i=sequence_length-1; i>= ((int) p_order)-1; i--)	//convert interval of size T
+		for (i=sequence_length-1; i>= p_order-1; i--)	//convert interval of size T
 		{
 			value=0;
-			for (j=i; j>=i-((int) p_order)+1; j--)
+			for (j=i; j>=i-p_order+1; j--)
 				value= (value >> max_val) | (obs[j] << (max_val * (p_order-1)));
 
 			obs[i]= (ST) value;
@@ -480,11 +643,14 @@ template <class ST> class CStringFeatures: public CFeatures
 
 		for (i=p_order-2;i>=0;i--)
 		{
+			if (i>=sequence_length)
+				continue;
+
 			value=0;
 			for (j=i; j>=i-p_order+1; j--)
 			{
 				value= (value >> max_val);
-				if (j>=0)
+				if (j>=0 && j<sequence_length)
 					value|=obs[j] << (max_val * (p_order-1));
 			}
 			obs[i]=value;
@@ -494,6 +660,39 @@ template <class ST> class CStringFeatures: public CFeatures
 			obs[i-start]=obs[i];
 	}
 
+	void translate_from_single_order_reversed(ST* obs, INT sequence_length, INT start, INT p_order, INT max_val)
+	{
+		INT i,j;
+		ST value=0;
+
+		for (i=sequence_length-1; i>= p_order-1; i--)	//convert interval of size T
+		{
+			value=0;
+			for (j=i; j>=i-p_order+1; j--)
+				value= (value << max_val) | obs[j];
+
+			obs[i]= (ST) value;
+		}
+
+		for (i=p_order-2;i>=0;i--)
+		{
+			if (i>=sequence_length)
+				continue;
+
+			value=0;
+			for (j=i; j>=i-p_order+1; j--)
+			{
+				value= (value << max_val);
+				if (j>=0 && j<sequence_length)
+					value|=obs[j];
+			}
+			obs[i]=value;
+		}
+
+		for (i=start; i<sequence_length; i++)	
+			obs[i-start]=obs[i];
+	}
+
 	void translate_from_single_order(ST* obs, INT sequence_length, INT start, INT p_order, INT max_val, INT gap)
 	{
 		ASSERT(gap>=0) ;
@@ -505,10 +704,10 @@ template <class ST> class CStringFeatures: public CFeatures
 		ST value=0;
 
 		// almost all positions
-		for (i=sequence_length-1; i>= ((int) p_order)-1; i--)	//convert interval of size T
+		for (i=sequence_length-1; i>=p_order-1; i--)	//convert interval of size T
 		{
 			value=0;
-			for (j=i; j>=i-((int) p_order)+1; j--)
+			for (j=i; j>=i-p_order+1; j--)
 			{
 				if (i-j<start_gap)
 				{
@@ -525,19 +724,22 @@ template <class ST> class CStringFeatures: public CFeatures
 		// the remaining `order` positions
 		for (i=p_order-2;i>=0;i--)
 		{
+			if (i>=sequence_length)
+				continue;
+
 			value=0;
 			for (j=i; j>=i-p_order+1; j--)
 			{
 				if (i-j<start_gap)
 				{
 					value= (value >> max_val);
-					if (j>=0)
+					if (j>=0 && j<sequence_length)
 						value|=obs[j] << (max_val * (p_order-1-gap));
 				}
 				else if (i-j>=end_gap)
 				{
 					value= (value >> max_val);
-					if (j>=0)
+					if (j>=0 && j<sequence_length)
 						value|=obs[j] << (max_val * (p_order-1-gap));
 				}			
 			}
@@ -548,6 +750,60 @@ template <class ST> class CStringFeatures: public CFeatures
 		for (i=start; i<sequence_length; i++)	
 			obs[i-start]=obs[i];
 	}
+	
+	void translate_from_single_order_reversed(ST* obs, INT sequence_length, INT start, INT p_order, INT max_val, INT gap)
+	{
+		ASSERT(gap>=0) ;
+		
+		const INT start_gap = (p_order - gap)/2 ;
+		const INT end_gap = start_gap + gap ;
+		
+		INT i,j;
+		ST value=0;
+
+		// almost all positions
+		for (i=sequence_length-1; i>=p_order-1; i--)	//convert interval of size T
+		{
+			value=0;
+			for (j=i; j>=i-p_order+1; j--)
+			{
+				if (i-j<start_gap)
+					value= (value << max_val) | obs[j];
+				else if (i-j>=end_gap)
+					value= (value << max_val) | obs[j];
+			}
+			obs[i]= (ST) value;
+		}
+
+		// the remaining `order` positions
+		for (i=p_order-2;i>=0;i--)
+		{
+			if (i>=sequence_length)
+				continue;
+
+			value=0;
+			for (j=i; j>=i-p_order+1; j--)
+			{
+				if (i-j<start_gap)
+				{
+					value= value << max_val;
+					if (j>=0 && j<sequence_length)
+						value|=obs[j];
+				}
+				else if (i-j>=end_gap)
+				{
+					value= value << max_val;
+					if (j>=0 && j<sequence_length)
+						value|=obs[j];
+				}			
+			}
+			obs[i]=value;
+		}
+
+		// shifting
+		for (i=start; i<sequence_length; i++)	
+			obs[i-start]=obs[i];
+	}
 
 	protected:
 
@@ -560,6 +816,12 @@ template <class ST> class CStringFeatures: public CFeatures
 	/// this contains the array of features.
 	T_STRING<ST>* features;
 
+	/// true when single string / created by sliding window
+	ST* single_string;
+
+	/// length of prior single string
+	INT length_of_single_string;
+
 	/// length of longest string
 	INT max_string_length;
 
