@@ -17,9 +17,11 @@
 
 #include "lib/common.h"
 #include "lib/io.h"
-#include <mex.h>
+#include "lib/matlab.h"
+
 #include <mexversion.c>
 
+#include "guilib/GUICommands.h"
 #include "guilib/GUIMatlab.h"
 #include "gui/TextGUI.h"
 #include "gui/GUI.h"
@@ -27,64 +29,6 @@
 static CGUIMatlab sg_matlab;
 extern CTextGUI* gui;
 
-static const CHAR* N_SEND_COMMAND=		"send_command";
-static const CHAR* N_HELP=		        "help";
-static const CHAR* N_CRC=			"crc";
-static const CHAR* N_TRANSLATE_STRING=			"translate_string";
-static const CHAR* N_GET_HMM=			"get_hmm";
-static const CHAR* N_GET_VITERBI_PATH=			"get_viterbi_path";
-static const CHAR* N_GET_SVM=			"get_svm";
-static const CHAR* N_GET_SVM_OBJECTIVE=		"get_svm_objective";
-static const CHAR* N_GET_KERNEL_INIT=	        "get_kernel_init";
-static const CHAR* N_GET_KERNEL_MATRIX=	        "get_kernel_matrix";
-static const CHAR* N_HMM_LIKELIHOOD=	        "hmm_likelihood";
-static const CHAR* N_GET_KERNEL_OPTIMIZATION=	        "get_kernel_optimization";
-static const CHAR* N_COMPUTE_BY_SUBKERNELS=	        "compute_by_subkernels";
-static const CHAR* N_SET_SUBKERNEL_WEIGHTS=	        "set_subkernel_weights";
-static const CHAR* N_SET_SUBKERNEL_WEIGHTS_COMBINED =	        "set_subkernel_weights_combined";
-static const CHAR* N_SET_LAST_SUBKERNEL_WEIGHTS=	        "set_last_subkernel_weights";
-static const CHAR* N_SET_WD_POS_WEIGHTS=	        "set_WD_position_weights";
-static const CHAR* N_GET_SUBKERNEL_WEIGHTS=	        "get_subkernel_weights";
-static const CHAR* N_GET_LAST_SUBKERNEL_WEIGHTS=	        "get_last_subkernel_weights";
-static const CHAR* N_GET_WD_POS_WEIGHTS=	        "get_WD_position_weights";
-static const CHAR* N_GET_WD_SCORING=	        "get_WD_scoring";
-static const CHAR* N_GET_FEATURES=		"get_features";
-static const CHAR* N_GET_LABELS=		"get_labels";
-static const CHAR* N_GET_VERSION=		"get_version";
-static const CHAR* N_GET_PREPROC_INIT=	        "get_preproc_init";
-static const CHAR* N_GET_HMM_DEFS=		"get_hmm_defs";
-static const CHAR* N_SET_HMM=			"set_hmm";
-//static const CHAR* N_MODEL_PROB_NO_B_TRANS=			"model_prob_no_b_trans";
-static const CHAR* N_BEST_PATH_NO_B_TRANS=			"best_path_no_b_trans";
-static const CHAR* N_BEST_PATH_TRANS_DERIV=			"best_path_trans_deriv";
-static const CHAR* N_BEST_PATH_TRANS=			"best_path_trans";
-static const CHAR* N_BEST_PATH_2STRUCT=			"best_path_2struct";
-static const CHAR* N_BEST_PATH_TRANS_SIMPLE=			"best_path_trans_simple";
-static const CHAR* N_BEST_PATH_NO_B=			"best_path_no_b";
-static const CHAR* N_APPEND_HMM=			"append_hmm";
-static const CHAR* N_SET_SVM=			"set_svm";
-static const CHAR* N_SET_CUSTOM_KERNEL=	        "set_custom_kernel";
-static const CHAR* N_SET_KERNEL_INIT=	        "set_kernel_init";
-static const CHAR* N_SET_FEATURES=		"set_features";
-static const CHAR* N_ADD_FEATURES=		"add_features";
-static const CHAR* N_SET_LABELS=		"set_labels";
-static const CHAR* N_SET_PREPROC_INIT=	        "set_preproc_init";
-static const CHAR* N_SET_HMM_DEFS=		"set_hmm_defs";
-static const CHAR* N_ONE_CLASS_HMM_CLASSIFY=		"one_class_hmm_classify";
-static const CHAR* N_ONE_CLASS_LINEAR_HMM_CLASSIFY=		"one_class_linear_hmm_classify";
-static const CHAR* N_HMM_CLASSIFY=		"hmm_classify";
-static const CHAR* N_ONE_CLASS_HMM_CLASSIFY_EXAMPLE=		"one_class_hmm_classify_example";
-static const CHAR* N_HMM_CLASSIFY_EXAMPLE=	"hmm_classify_example";
-static const CHAR* N_RELATIVE_ENTROPY =	"relative_entropy";
-static const CHAR* N_ENTROPY =	"entropy";
-static const CHAR* N_CLASSIFY=		"classify";
-static const CHAR* N_SVM_CLASSIFY=		"svm_classify";
-static const CHAR* N_SVM_CLASSIFY_EXAMPLE=	"svm_classify_example";
-static const CHAR* N_GET_PLUGIN_ESTIMATE=	"get_plugin_estimate";
-static const CHAR* N_SET_PLUGIN_ESTIMATE=	"set_plugin_estimate";
-static const CHAR* N_PLUGIN_ESTIMATE_CLASSIFY=	"plugin_estimate_classify";
-static const CHAR* N_PLUGIN_ESTIMATE_CLASSIFY_EXAMPLE=	"plugin_estimate_classify_example";
-
 /// return true if str starts with cmd
 /// cmd is a 0 terminated string const
 /// str is a string of length len (not 0 terminated)
@@ -99,8 +43,8 @@ void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
 	if (!gui)
 		gui=new CTextGUI(0, NULL);
 
-#ifndef CYGWIN
-	CSignal::set_handler();
+#ifndef WIN32
+    CSignal::set_handler();
 #endif
 
 	if (!gui)
@@ -127,19 +71,15 @@ void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
 		}
 		else if (strmatch(action, len, N_HELP))
 		{
-			if (nrhs==1)
-			{
-				sg_matlab.send_command("help");
-			}
+			if (nrhs==1 && nlhs==0)
+				gui->print_help();
 			else
 				SG_SERROR( "usage is sg('help')");
 		}
 		else if (strmatch(action, len, N_GET_HMM))
 		{
 			if (nlhs==4)
-			{
 				sg_matlab.get_hmm(plhs);
-			}
 			else
 				SG_SERROR( "usage is [p,q,a,b]=sg('get_hmm')");
 		}
@@ -210,18 +150,14 @@ void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
 		else if (strmatch(action, len, N_RELATIVE_ENTROPY))
 		{
 			if (nlhs==1 && nrhs==1)
-			{
-					sg_matlab.relative_entropy(plhs);
-			}
+				sg_matlab.relative_entropy(plhs);
 			else
 				SG_SERROR( "usage is [result]=sg('relative_entropy')");
 		}
 		else if (strmatch(action, len, N_ENTROPY))
 		{
 			if (nlhs==1 && nrhs==1)
-			{
-					sg_matlab.entropy(plhs);
-			}
+				sg_matlab.entropy(plhs);
 			else
 				SG_SERROR( "usage is [result]=sg('permuation_entropy')");
 		}
@@ -235,38 +171,40 @@ void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
 		else if (strmatch(action, len, N_GET_SVM_OBJECTIVE))
 		{
 			if (nlhs==1)
-			{
 				sg_matlab.get_svm_objective(plhs);
-			}
 			else
 				SG_SERROR( "usage is [obj]=sg('get_svm_objective')");
 		}
+		else if (strmatch(action, len, N_GET_CLASSIFIER))
+		{
+			if (nlhs==2)
+				sg_matlab.get_classifier(plhs);
+			else
+				SG_SERROR( "usage is [bias,weights]=sg('get_classifier')");
+		}
 		else if (strmatch(action, len, N_GET_SVM))
 		{
 			if (nlhs==2)
-			{
-				sg_matlab.get_svm(plhs);
-			}
+				sg_matlab.get_classifier(plhs);
 			else
 				SG_SERROR( "usage is [b,alphas]=sg('get_svm')");
 		}
 		else if (strmatch(action, len, N_SET_SVM))
 		{
 			if (nrhs==3)
-			{
 				sg_matlab.set_svm(prhs);
-			}
 			else
 				SG_SERROR( "usage is sg('set_svm', [ b, alphas])");
 		}
-		else if (strmatch(action, len, N_SVM_CLASSIFY_EXAMPLE))
+		else if (strmatch(action, len, N_CLASSIFY_EXAMPLE) ||
+				strmatch(action, len, N_SVM_CLASSIFY_EXAMPLE))
 		{
 			if (nlhs==1 && nrhs==2)
 			{
 				if (mxIsDouble(prhs[1]))
 				{
 				  double* idx=mxGetPr(prhs[1]);
-				  if (!sg_matlab.svm_classify_example(plhs, (int) (*idx) ))
+				  if (!sg_matlab.classify_example(plhs, (int) (*idx) ))
 				    SG_SERROR( "svm_classify_example failed");
 				}
 				else
@@ -275,21 +213,11 @@ void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
 			else
 				SG_SERROR( "usage is [result]=sg('svm_classify_example', feature_vector_index)");
 		}
-		else if (strmatch(action, len, N_CLASSIFY))
+		else if (strmatch(action, len, N_CLASSIFY) || strmatch(action, len, N_SVM_CLASSIFY))
 		{
 			if (nlhs==1)
 			  {
 			    if (!sg_matlab.classify(plhs))
-			      SG_SERROR( "classify failed");
-			  }
-			else
-				SG_SERROR( "usage is [result]=sg('classify')");
-		}
-		else if (strmatch(action, len, N_SVM_CLASSIFY))
-		{
-			if (nlhs==1)
-			  {
-			    if (!sg_matlab.svm_classify(plhs))
 			      SG_SERROR( "svm_classify failed");
 			  }
 			else
@@ -298,18 +226,14 @@ void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
 		else if (strmatch(action, len, N_GET_PLUGIN_ESTIMATE))
 		{
 			if (nlhs==2)
-			{
 				sg_matlab.get_plugin_estimate(plhs);
-			}
 			else
 				SG_SERROR( "usage is [emission_probs, model_sizes]=sg('get_plugin_estimate')");
 		}
 		else if (strmatch(action, len, N_SET_PLUGIN_ESTIMATE))
 		{
 			if (nrhs==3)
-			{
 				sg_matlab.set_plugin_estimate(prhs);
-			}
 			else
 				SG_SERROR( "usage is sg('set_plugin_estimate', emission_probs, model_sizes)");
 		}
@@ -340,9 +264,9 @@ void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
 		}
 		else if (strmatch(action, len, N_GET_KERNEL_OPTIMIZATION))
 		{
-			if ((nlhs==1) && (nrhs==1))
+			if ((nlhs==1) && (nrhs>=1))
 			{
-				if (!sg_matlab.get_kernel_optimization(plhs))
+				if (!sg_matlab.get_kernel_optimization(plhs, prhs, nrhs))
 					SG_SERROR( "error executing command");
 			}
 			else
@@ -402,6 +326,50 @@ void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
 			else
 				SG_SERROR( "usage is W=sg('get_WD_scoring', max_order)");
 		}
+		else if (strmatch(action, len, N_COMPUTE_POIM_WD))
+		{
+		    if (nlhs==1 && nrhs==3)
+			{
+				if (!sg_matlab.compute_poim_wd(plhs, prhs, nrhs))
+					SG_SERROR( "error executing command");
+			}
+			else
+				SG_SERROR( "usage is W=sg('compute_poim_wd', max_order, distribution)");
+		}
+		else if (strmatch(action, len, N_GET_WD_CONSENSUS))
+		{
+		    if (nlhs==1 && nrhs==1)
+			{
+				if (!sg_matlab.get_WD_consensus(plhs))
+					SG_SERROR( "error executing command");
+			}
+			else
+				SG_SERROR( "usage is W=sg('get_WD_consensus')");
+		}
+		else if (strmatch(action, len, N_GET_SPEC_SCORING))
+		{
+		    if (nlhs==1 && nrhs==2)
+			{
+				if (mxIsDouble(prhs[1]))
+				{
+					double* idx=mxGetPr(prhs[1]);
+					if (!sg_matlab.get_SPEC_scoring(plhs, (INT) (*idx)))
+						SG_SERROR( "error executing command");
+				}
+			}
+			else
+				SG_SERROR( "usage is W=sg('get_SPEC_scoring', max_order)");
+		}
+		else if (strmatch(action, len, N_GET_SPEC_CONSENSUS))
+		{
+		    if (nlhs==1 && nrhs==1)
+			{
+				if (!sg_matlab.get_SPEC_consensus(plhs))
+					SG_SERROR( "error executing command");
+			}
+			else
+				SG_SERROR( "usage is W=sg('get_SPEC_consensus')");
+		}
 		else if (strmatch(action, len, N_SET_LAST_SUBKERNEL_WEIGHTS))
 		{
 			if ((nlhs==0) && (nrhs==2))
@@ -440,8 +408,13 @@ void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
 				if (!sg_matlab.set_WD_position_weights(prhs[1]))
 					SG_SERROR( "error executing command");
 			}
+			else if ((nlhs==0) && (nrhs==3))
+			{
+				if (!sg_matlab.set_WD_position_weights_per_example(prhs[1], prhs[2]))
+					SG_SERROR( "error executing command");
+			}
 			else
-				SG_SERROR( "usage is sg('set_WD_position_weights', W)");
+				SG_SERROR( "usage is sg('set_WD_position_weights', W[, 'TRAIN'|'TEST'])");
 		}
 		else if (strmatch(action, len, N_GET_KERNEL_MATRIX))
 		{
@@ -450,6 +423,13 @@ void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
 			else
 				SG_SERROR( "usage is K=sg('get_kernel_matrix')");
 		}
+		else if (strmatch(action, len, N_GET_DISTANCE_MATRIX))
+		{
+			if ((nlhs==1) && (nrhs==1))
+				sg_matlab.get_distance_matrix(plhs);
+			else
+				SG_SERROR( "usage is D=sg('get_distance_matrix')");
+		}
 		else if (strmatch(action, len, N_GET_KERNEL_INIT))
 		{
 		}
@@ -532,36 +512,28 @@ void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
 		else if (strmatch(action, len, N_BEST_PATH_TRANS_SIMPLE))
 		{
 			if ((nrhs==1+5) & (nlhs==2))
-			{
 				sg_matlab.best_path_trans_simple(prhs,plhs);
-			}
 			else
 				SG_SERROR( "usage is [prob,path]=sg('best_path_trans_simple', p, q, a_trans, seq, nbest)");
 		}
 		else if (strmatch(action, len, N_BEST_PATH_TRANS_DERIV))
 		{
 			if (((nrhs==1+13) || (nrhs==1+15)) & ((nlhs==5)||(nlhs==6)))
-			{
 				sg_matlab.best_path_trans_deriv(prhs, nrhs, plhs, nlhs);
-			}
 			else
 				SG_SERROR( "usage is [p_deriv, q_deriv, a_deriv, penalties_deriv, my_scores, my_loss]=sg('best_path_trans_deriv', my_path, my_pos, p,q,a_trans,seq, pos, genestr, penalties, state_signals, penalty_info, dict_weights, mod_words [, segment_loss, segment_ids_mask]) [%i,%i]", nrhs, nlhs);
 		}
 		else if (strmatch(action, len, N_BEST_PATH_TRANS))
 		{
 			if ( ((nrhs==1+14) || (nrhs==1+16)) && (nlhs==3) )
-			{
 				sg_matlab.best_path_trans(prhs, nrhs, plhs);
-			}
 			else
 				SG_SERROR( "usage is [prob,path,pos]=sg('best_path_trans',p,q,a_trans,seq,pos,orf_info, genestr, penalties, state_signals, penalty_info, nbest, dict_weights, use_orf, mod_words [, segment_loss, segment_ids_mask])");
 		}
 		else if (strmatch(action, len, N_BEST_PATH_2STRUCT))
 		{
 			if ((nrhs==1+11) & (nlhs==3))
-			{
 				sg_matlab.best_path_2struct(prhs,plhs);
-			}
 			else
 				SG_SERROR( "usage is [prob,path,pos]=sg('best_path_2struct',p,q,a_trans,seq,pos, genestr, penalties, penalty_info, nbest, dict_weights, segment_sum_weights)");
 		}
@@ -577,36 +549,28 @@ void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
 		else if (strmatch(action, len, N_BEST_PATH_NO_B))
 		{
 			if ((nrhs==1+4) & (nlhs==2))
-			{
 				sg_matlab.best_path_no_b(prhs,plhs);
-			}
 			else
 				SG_SERROR( "usage is [prob,path]=sg('best_path_no_b',p,q,a,max_iter)");
 		}
 		else if (strmatch(action, len, N_SET_HMM))
 		{
 			if (nrhs==1+4)
-			{
 				sg_matlab.set_hmm(prhs);
-			}
 			else
 				SG_SERROR( "usage is sg('set_hmm',[p,q,a,b])");
 		}
 		else if (strmatch(action, len, N_APPEND_HMM))
 		{
 			if (nrhs==1+4)
-			{
 				sg_matlab.append_hmm(prhs);
-			}
 			else
 				SG_SERROR( "usage is sg('append_hmm',[p,q,a,b])");
 		}
 		else if (strmatch(action, len, N_SET_SVM))
 		{
 			if (nrhs==1+2)
-			{
 				sg_matlab.set_svm(prhs);
-			}
 			else
 				SG_SERROR( "usage is sg('set_svm',[b,alphas])");
 		}
@@ -677,6 +641,16 @@ void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
 				SG_SERROR( "usage is sg('set_features', 'TRAIN|TEST', features, ...)");
 			SG_SINFO( "done\n");
 		}
+		else if (strmatch(action, len, N_FROM_POSITION_LIST))
+		{
+			if ((nlhs==0) && (nrhs>=4))
+			{
+				if (!sg_matlab.from_position_list(prhs, nrhs))
+					SG_SERROR( "error occured in obtain_from_position_list command");
+			}
+			else
+				SG_SERROR( "usage is sg('from_position_list', 'TRAIN|TEST', <[list]>, [skip])");
+		}
 		else if (strmatch(action, len, N_ADD_FEATURES))
 		{
 			if (nrhs>=3)
@@ -848,8 +822,8 @@ void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
 	else
 		SG_SERROR( "string expected as first argument");
 
-#ifndef CYGWIN
-	CSignal::unset_handler();
+#ifndef WIN32
+    CSignal::unset_handler();
 #endif
 }
 #endif //HAVE_SWIG
