@@ -13,6 +13,7 @@
 
 #ifndef HAVE_SWIG
 #include "gui/TextGUI.h"
+#include "guilib/GUICommands.h"
 
 #include <ctype.h>
 #include <sys/types.h>
@@ -47,107 +48,6 @@ CHistogram* h;
 #include <libmmfile.h>
 #endif // WITHMATLAB
 
-//names of menu commands
-static const CHAR* N_NEW_HMM=			"new_hmm";
-static const CHAR* N_NEW_SVM=			"new_svm";
-static const CHAR* N_NEW_CLASSIFIER=	"new_classifier";
-static const CHAR* N_NEW_KNN=			"new_knn";
-static const CHAR* N_NEW_PLUGIN_ESTIMATOR="new_plugin_estimator";
-static const CHAR* N_TRAIN_ESTIMATOR=	"train_estimator";
-static const CHAR* N_TRAIN_CLASSIFIER=	"train_classifier";
-static const CHAR* N_SET_PERCEPTRON_PARAMETERS=	"set_perceptron_parameters";
-static const CHAR* N_TEST_ESTIMATOR=	"test_estimator";
-static const CHAR* N_TRAIN_KNN=	"train_knn";
-static const CHAR* N_TEST_KNN=	"test_knn";
-static const CHAR* N_LOAD_PREPROC=		"load_preproc";
-static const CHAR* N_SAVE_PREPROC=		"save_preproc";
-static const CHAR* N_LOAD_HMM=			"load_hmm";
-static const CHAR* N_SAVE_HMM=			"save_hmm";
-static const CHAR* N_LOAD_SVM=			"load_svm";
-static const CHAR* N_SAVE_SVM=			"save_svm";
-static const CHAR* N_LOAD_KERNEL_INIT=	"load_kernel_init";
-static const CHAR* N_SAVE_KERNEL_INIT=	"save_kernel_init";
-static const CHAR* N_LOAD_LABELS=		"load_labels";
-static const CHAR* N_LOAD_FEATURES=		"load_features";
-static const CHAR* N_SAVE_FEATURES=		"save_features";
-static const CHAR* N_CLEAN_FEATURES=	"clean_features";
-static const CHAR* N_RESHAPE=			"reshape";
-static const CHAR* N_LOAD_DEFINITIONS=	"load_defs";
-static const CHAR* N_SAVE_KERNEL=		"save_kernel";
-static const CHAR* N_SET_HMM_AS=		"set_hmm_as";
-static const CHAR* N_NORMALIZE=			"normalize_hmm";
-static const CHAR* N_RELATIVE_ENTROPY=	"relative_entropy";
-static const CHAR* N_ENTROPY=			"entropy";
-static const CHAR* N_PERMUTATION_ENTROPY="permutation_entropy";
-static const CHAR* N_SET_KERNEL=		"set_kernel";
-static const CHAR* N_SET_DISTANCE=		"set_distance";
-static const CHAR* N_ADD_KERNEL=		"add_kernel";
-static const CHAR* N_CLEAN_KERNEL=		"clean_kernel";
-
-static const CHAR* N_SET_KERNEL_OPTIMIZATION_TYPE=		"set_kernel_optimization_type";
-static const CHAR* N_ATTACH_PREPROC=	"attach_preproc";
-static const CHAR* N_ADD_PREPROC=		"add_preproc";
-static const CHAR* N_DEL_PREPROC=		"del_preproc";
-static const CHAR* N_CLEAN_PREPROC=		"clean_preproc";
-static const CHAR* N_INIT_KERNEL=		"init_kernel";
-static const CHAR* N_INIT_DISTANCE=		"init_distance";
-static const CHAR* N_DELETE_KERNEL_OPTIMIZATION=		"delete_kernel_optimization";
-static const CHAR* N_INIT_KERNEL_OPTIMIZATION  =		"init_kernel_optimization";
-static const CHAR* N_SAVE_PATH=			"save_hmm_path";
-static const CHAR* N_SAVE_LIKELIHOOD=	"save_hmm_likelihood";
-static const CHAR* N_BEST_PATH=			"best_path";
-static const CHAR* N_VITERBI_TRAIN=	       	"vit";
-static const CHAR* N_VITERBI_TRAIN_DEFINED=     "vit_def";
-static const CHAR* N_LINEAR_TRAIN=       	"linear_train";
-static const CHAR* N_CLEAR=			"clear";
-static const CHAR* N_CHOP=			"chop";
-static const CHAR* N_CONVERGENCE_CRITERIA=	"convergence_criteria";
-static const CHAR* N_PSEUDO=			"pseudo";
-static const CHAR* N_CONVERT=	"convert";
-static const CHAR* N_C=			     	"c";
-static const CHAR* N_LOGLEVEL=			     	"loglevel";
-static const CHAR* N_ECHO=			     	"echo";
-static const CHAR* N_SVMQPSIZE=			     	"svm_qpsize";
-static const CHAR* N_THREADS=			     	"threads";
-static const CHAR* N_MKL_PARAMETERS=			"mkl_parameters";
-static const CHAR* N_SVM_EPSILON=			"svm_epsilon";
-static const CHAR* N_SVM_MAX_TRAIN_TIME=		"svm_max_train_time";
-static const CHAR* N_SVR_TUBE_EPSILON=			"svr_tube_epsilon";
-static const CHAR* N_SVM_ONE_CLASS_NU=			"svm_one_class_nu";
-static const CHAR* N_SVM_TRAIN_AUC_MAXIMIZATION=			"svm_train_auc_maximization";
-static const CHAR* N_ADD_STATES=	        "add_states";
-static const CHAR* N_APPEND_HMM=		"append_hmm";
-static const CHAR* N_BAUM_WELCH_TRAIN=	        "bw";
-static const CHAR* N_BAUM_WELCH_TRANS_TRAIN=	"bw_trans";
-static const CHAR* N_BAUM_WELCH_TRAIN_DEFINED=	"bw_def";
-static const CHAR* N_LIKELIHOOD=	       	"likelihood";
-static const CHAR* N_ALPHABET=			"alphabet";
-static const CHAR* N_USE_BATCH_COMPUTATION =			"use_batch_computation";
-static const CHAR* N_USE_MKL =			"use_mkl";
-static const CHAR* N_USE_SHRINKING=			"use_shrinking";
-static const CHAR* N_USE_LINADD=			"use_linadd";
-static const CHAR* N_USE_PRECOMPUTE=			"use_precompute";
-static const CHAR* N_OUTPUT_HMM=		"output_hmm";
-static const CHAR* N_OUTPUT_HMM_DEFINED=        "output_hmm_defined";
-static const CHAR* N_QUIT=			"quit";
-static const CHAR* N_EXEC=			"exec";
-static const CHAR* N_EXIT=			"exit";
-static const CHAR* N_HELP=			"help";
-static const CHAR* N_SYSTEM=			"!";
-static const CHAR N_COMMENT1=			'#';
-static const CHAR N_COMMENT2=			'%';
-static const CHAR* N_SET_MAX_DIM=		"max_dim";
-static const CHAR* N_SET_THRESHOLD=			"set_threshold";
-static const CHAR* N_SVM_TRAIN=			"svm_train";
-static const CHAR* N_SVM_TEST=			"svm_test";
-static const CHAR* N_ONE_CLASS_HMM_TEST=	"one_class_hmm_test";
-static const CHAR* N_HMM_TEST=			"hmm_test";
-static const CHAR* N_HMM_CLASSIFY=		"hmm_classify";
-static const CHAR* N_SET_OUTPUT=		"set_output";
-static const CHAR* N_SET_REF_FEAT=              "set_ref_features" ;
-static const CHAR* N_TIC=              "tic" ;
-static const CHAR* N_TOC=              "toc" ;
-
 CTextGUI::CTextGUI(INT p_argc, char** p_argv)
 : CGUI(p_argc, p_argv), out_file(NULL), echo(true)
 {
@@ -220,7 +120,8 @@ void CTextGUI::print_help()
 	SG_PRINT( "\033[1;31m%s\033[0m\t <LIGHT|LIBSVM> - creates SVM of type LIGHT or LIBSVM\n",N_NEW_SVM);
 	SG_PRINT( "\033[1;31m%s\033[0m [c-value]\t\t\t- changes svm_c value\n", N_C);
 	SG_PRINT( "\033[1;31m%s\033[0m [qpsize]\t\t\t- changes svm_qpsize value\n", N_SVMQPSIZE);
-	SG_PRINT( "\033[1;31m%s\033[0m [threads]\t\t\t- changes svm_threads value\n", N_SVMQPSIZE);
+	SG_PRINT( "\033[1;31m%s\033[0m [bufsize]\t\t\t- changes svm_bufsize value\n", N_SVMBUFSIZE);
+	SG_PRINT( "\033[1;31m%s\033[0m [threads]\t\t\t- changes number of threads\n", N_THREADS);
 	SG_PRINT( "\033[1;31m%s\033[0m [epsilon-value]\t\t\t- changes svm-epsilon value\n", N_SVM_EPSILON);
 	SG_PRINT( "\033[1;31m%s\033[0m [seconds]\t\t\t- changes svm max training time\n", N_SVM_MAX_TRAIN_TIME);
 	SG_PRINT( "\033[1;31m%s\033[0m [epsilon-value]\t\t\t- changes svr-tube-epsilon value\n", N_SVR_TUBE_EPSILON);
@@ -318,7 +219,7 @@ bool CTextGUI::parse_line(CHAR* p_input)
 	} 
 	else if (!strncmp(p_input, N_NEW_SVM, strlen(N_NEW_SVM)))
 	{
-		guisvm.new_svm(p_input+strlen(N_NEW_SVM));
+		guiclassifier.new_classifier(p_input+strlen(N_NEW_SVM));
 	} 
 	else if (!strncmp(p_input, N_NEW_CLASSIFIER, strlen(N_NEW_CLASSIFIER)))
 	{
@@ -372,6 +273,10 @@ bool CTextGUI::parse_line(CHAR* p_input)
 	{
 		guifeatures.clean(p_input+strlen(N_CLEAN_FEATURES));
 	} 
+	else if (!strncmp(p_input, N_SLIDE_WINDOW, strlen(N_SLIDE_WINDOW)))
+	{
+		guifeatures.slide_window(p_input+strlen(N_SLIDE_WINDOW));
+	} 
 	else if (!strncmp(p_input, N_RESHAPE, strlen(N_RESHAPE)))
 	{
 		guifeatures.reshape(p_input+strlen(N_RESHAPE));
@@ -386,7 +291,7 @@ bool CTextGUI::parse_line(CHAR* p_input)
 	} 
 	else if (!strncmp(p_input, N_LOAD_SVM, strlen(N_LOAD_SVM)))
 	{
-		guisvm.load(p_input+strlen(N_LOAD_SVM));
+		guiclassifier.load(p_input+strlen(N_LOAD_SVM));
 	} 
 	else if (!strncmp(p_input, N_DELETE_KERNEL_OPTIMIZATION, strlen(N_DELETE_KERNEL_OPTIMIZATION)))
 	{
@@ -434,7 +339,7 @@ bool CTextGUI::parse_line(CHAR* p_input)
 	} 
 	else if (!strncmp(p_input, N_SAVE_SVM, strlen(N_SAVE_SVM)))
 	{
-		guisvm.save(p_input+strlen(N_SAVE_SVM)) ;
+		guiclassifier.save(p_input+strlen(N_SAVE_SVM)) ;
 	} 
 	else if (!strncmp(p_input, N_SAVE_KERNEL_INIT, strlen(N_SAVE_KERNEL_INIT)))
 	{
@@ -562,9 +467,9 @@ bool CTextGUI::parse_line(CHAR* p_input)
 	{
 		guihmm.linear_train(p_input+strlen(N_LINEAR_TRAIN));
 	} 
-	else if (!strncmp(p_input, N_SVM_TRAIN_AUC_MAXIMIZATION, strlen(N_SVM_TRAIN_AUC_MAXIMIZATION)))
+	else if (!strncmp(p_input, N_DO_AUC_MAXIMIZATION, strlen(N_DO_AUC_MAXIMIZATION)))
 	{
-		guisvm.train(p_input+strlen(N_SVM_TRAIN_AUC_MAXIMIZATION), true);
+		guiclassifier.set_do_auc_maximization(p_input+strlen(N_DO_AUC_MAXIMIZATION));
 	} 
 	else if (!strncmp(p_input, N_TRAIN_CLASSIFIER, strlen(N_TRAIN_CLASSIFIER)))
 	{
@@ -576,7 +481,7 @@ bool CTextGUI::parse_line(CHAR* p_input)
 	} 
 	else if (!strncmp(p_input, N_SVM_TRAIN, strlen(N_SVM_TRAIN)))
 	{
-		guisvm.train(p_input+strlen(N_SVM_TRAIN), false);
+		guiclassifier.train(p_input+strlen(N_SVM_TRAIN));
 	} 
 	else if (!strncmp(p_input, N_SET_KERNEL_OPTIMIZATION_TYPE, strlen(N_SET_KERNEL_OPTIMIZATION_TYPE)))
 	{
@@ -617,7 +522,7 @@ bool CTextGUI::parse_line(CHAR* p_input)
 	} 
 	else if (!strncmp(p_input, N_SVM_TEST, strlen(N_SVM_TEST)))
 	{
-		guisvm.test(p_input+strlen(N_SVM_TEST));
+		guiclassifier.test(p_input+strlen(N_SVM_TEST));
 	} 
 	else if (!strncmp(p_input, N_ONE_CLASS_HMM_TEST, strlen(N_ONE_CLASS_HMM_TEST)))
 	{
@@ -657,11 +562,19 @@ bool CTextGUI::parse_line(CHAR* p_input)
 	} 
 	else if (!strncmp(p_input, N_C, strlen(N_C)))
 	{
-		guisvm.set_C(p_input+strlen(N_C));
+		guiclassifier.set_svm_C(p_input+strlen(N_C));
 	} 
 	else if (!strncmp(p_input, N_SVMQPSIZE, strlen(N_SVMQPSIZE)))
 	{
-		guisvm.set_qpsize(p_input+strlen(N_SVMQPSIZE));
+		guiclassifier.set_svm_qpsize(p_input+strlen(N_SVMQPSIZE));
+	} 
+	else if (!strncmp(p_input, N_SVMBUFSIZE, strlen(N_SVMBUFSIZE)))
+	{
+		guiclassifier.set_svm_bufsize(p_input+strlen(N_SVMBUFSIZE));
+	} 
+	else if (!strncmp(p_input, N_SVMMAXQPSIZE, strlen(N_SVMMAXQPSIZE)))
+	{
+		guiclassifier.set_svm_max_qpsize(p_input+strlen(N_SVMMAXQPSIZE));
 	} 
 	else if (!strncmp(p_input, N_THREADS, strlen(N_THREADS)))
 	{
@@ -672,50 +585,54 @@ bool CTextGUI::parse_line(CHAR* p_input)
 
 		sscanf(param, "%d", &threads) ;
 
-		CParallel::set_num_threads(threads);
+		parallel.set_num_threads(threads);
 
 		SG_INFO( "Set number of threads to %d\n", threads);
 		return true ;  
 	} 
 	else if (!strncmp(p_input, N_USE_PRECOMPUTE, strlen(N_USE_PRECOMPUTE)))
 	{
-		guisvm.set_precompute_enabled(p_input+strlen(N_USE_PRECOMPUTE));
+		guiclassifier.set_svm_precompute_enabled(p_input+strlen(N_USE_PRECOMPUTE));
 	} 
 	else if (!strncmp(p_input, N_USE_MKL, strlen(N_USE_MKL)))
 	{
-		guisvm.set_mkl_enabled(p_input+strlen(N_USE_MKL));
+		guiclassifier.set_svm_mkl_enabled(p_input+strlen(N_USE_MKL));
 	} 
 	else if (!strncmp(p_input, N_USE_SHRINKING, strlen(N_USE_SHRINKING)))
 	{
-		guisvm.set_shrinking_enabled(p_input+strlen(N_USE_SHRINKING));
+		guiclassifier.set_svm_shrinking_enabled(p_input+strlen(N_USE_SHRINKING));
 	} 
 	else if (!strncmp(p_input, N_USE_BATCH_COMPUTATION, strlen(N_USE_BATCH_COMPUTATION)))
 	{
-		guisvm.set_batch_computation_enabled(p_input+strlen(N_USE_BATCH_COMPUTATION));
+		guiclassifier.set_svm_batch_computation_enabled(p_input+strlen(N_USE_BATCH_COMPUTATION));
 	} 
 	else if (!strncmp(p_input, N_USE_LINADD, strlen(N_USE_LINADD)))
 	{
-		guisvm.set_linadd_enabled(p_input+strlen(N_USE_LINADD));
+		guiclassifier.set_svm_linadd_enabled(p_input+strlen(N_USE_LINADD));
 	} 
 	else if (!strncmp(p_input, N_SVM_EPSILON, strlen(N_SVM_EPSILON)))
 	{
-		guisvm.set_svm_epsilon(p_input+strlen(N_SVM_EPSILON));
+		guiclassifier.set_svm_epsilon(p_input+strlen(N_SVM_EPSILON));
 	} 
 	else if (!strncmp(p_input, N_SVM_MAX_TRAIN_TIME, strlen(N_SVM_MAX_TRAIN_TIME)))
 	{
-		guisvm.set_svm_max_train_time(p_input+strlen(N_SVM_MAX_TRAIN_TIME));
+		guiclassifier.set_max_train_time(p_input+strlen(N_SVM_MAX_TRAIN_TIME));
 	} 
 	else if (!strncmp(p_input, N_SVR_TUBE_EPSILON, strlen(N_SVR_TUBE_EPSILON)))
 	{
-		guisvm.set_svr_tube_epsilon(p_input+strlen(N_SVR_TUBE_EPSILON));
+		guiclassifier.set_svr_tube_epsilon(p_input+strlen(N_SVR_TUBE_EPSILON));
+	} 
+	else if (!strncmp(p_input, N_SVM_USE_BIAS, strlen(N_SVM_USE_BIAS)))
+	{
+		guiclassifier.set_svm_bias_enabled(p_input+strlen(N_SVM_USE_BIAS));
 	} 
 	else if (!strncmp(p_input, N_SVM_ONE_CLASS_NU, strlen(N_SVM_ONE_CLASS_NU)))
 	{
-		guisvm.set_svm_one_class_nu(p_input+strlen(N_SVM_ONE_CLASS_NU));
+		guiclassifier.set_svm_one_class_nu(p_input+strlen(N_SVM_ONE_CLASS_NU));
 	} 
 	else if (!strncmp(p_input, N_MKL_PARAMETERS, strlen(N_MKL_PARAMETERS)))
 	{
-		guisvm.set_mkl_parameters(p_input+strlen(N_MKL_PARAMETERS));
+		guiclassifier.set_svm_mkl_parameters(p_input+strlen(N_MKL_PARAMETERS));
 	} 
 	else if (!strncmp(p_input, N_TIC, strlen(N_TIC)))
 	{
