@@ -18,7 +18,7 @@
 #include "lib/io.h"
 
 	CGUIFeatures::CGUIFeatures(CGUI * gui_)
-: gui(gui_), train_features(NULL), test_features(NULL), ref_features(NULL)
+: CSGObject(), gui(gui_), train_features(NULL), test_features(NULL), ref_features(NULL)
 {
 }
 
@@ -182,33 +182,30 @@ bool CGUIFeatures::save(CHAR* param)
 
 		if (*f_ptr)
 		{
-			if (fname)
+			if (strcmp(type,"REAL")==0)
 			{
-				if (strcmp(type,"REAL")==0)
-				{
-					result= ((CRealFeatures*) (*f_ptr))->save(fname);
-				}
-				else if (strcmp(type, "BYTE")==0)
-				{
-					result= ((CByteFeatures*) (*f_ptr))->save(fname);
-				}
-				else if (strcmp(type, "CHAR")==0)
-				{
-					result= ((CCharFeatures*) (*f_ptr))->save(fname);
-				}
-				else if (strcmp(type, "SHORT")==0)
-				{
-					result= ((CShortFeatures*) (*f_ptr))->save(fname);
-				}
-				else if (strcmp(type, "WORD")==0)
-				{
-					result= ((CWordFeatures*) (*f_ptr))->save(fname);
-				}
-				else
-				{
-					SG_ERROR( "unknown type\n");
-					return false;
-				}
+				result= ((CRealFeatures*) (*f_ptr))->save(fname);
+			}
+			else if (strcmp(type, "BYTE")==0)
+			{
+				result= ((CByteFeatures*) (*f_ptr))->save(fname);
+			}
+			else if (strcmp(type, "CHAR")==0)
+			{
+				result= ((CCharFeatures*) (*f_ptr))->save(fname);
+			}
+			else if (strcmp(type, "SHORT")==0)
+			{
+				result= ((CShortFeatures*) (*f_ptr))->save(fname);
+			}
+			else if (strcmp(type, "WORD")==0)
+			{
+				result= ((CWordFeatures*) (*f_ptr))->save(fname);
+			}
+			else
+			{
+				SG_ERROR( "unknown type\n");
+				return false;
 			}
 
 			if (!result)
@@ -252,6 +249,67 @@ bool CGUIFeatures::clean(CHAR* param)
 	return false ;
 }
 
+bool CGUIFeatures::slide_window(CHAR* param)
+{
+	param=CIO::skip_spaces(param);
+	INT shift=-1;
+	INT winsize=-1;
+	INT skip=0;
+	CHAR target[1024]="";
+
+	if ((sscanf(param, "%s %d %d %d", target, &winsize, &shift, &skip))>=3)
+	{
+		ASSERT(winsize>0);
+		ASSERT(shift>0);
+
+		CFeatures* features=NULL;
+
+		if (strcmp(target,"TRAIN")==0)
+		{
+			invalidate_train();
+			features=train_features;
+		}
+		else if (strcmp(target,"TEST")==0)
+		{
+			invalidate_test();
+			features=test_features;
+		}
+		else
+		{
+			SG_ERROR( "see help for parameters\n");
+			return false;
+		}
+
+		if (((CFeatures*) features)->get_feature_class() == C_COMBINED)
+		{
+			features=((CCombinedFeatures*) features)->get_last_feature_obj();
+			ASSERT(features);
+		}
+
+		ASSERT(features);
+		ASSERT(((CFeatures*) features)->get_feature_class() == C_STRING);
+
+		switch (features->get_feature_type())
+		{
+			case F_CHAR:
+				return ( ((CStringFeatures<CHAR>*) features)->obtain_by_sliding_window(winsize, shift, skip) > 0);
+			case F_BYTE:
+				return ( ((CStringFeatures<BYTE>*) features)->obtain_by_sliding_window(winsize, shift, skip) > 0);
+			case F_WORD:
+				return ( ((CStringFeatures<WORD>*) features)->obtain_by_sliding_window(winsize, shift, skip) > 0);
+			case F_ULONG:
+				return ( ((CStringFeatures<ULONG>*) features)->obtain_by_sliding_window(winsize, shift, skip) > 0);
+			default:
+				SG_SERROR("unsupported string features type\n");
+				return false;
+		}
+	}
+	else
+		SG_ERROR( "see help for params\n");
+
+	return false;
+}
+
 bool CGUIFeatures::reshape(CHAR* param)
 {
 	bool result=false;
@@ -317,37 +375,44 @@ CSparseFeatures<DREAL>* CGUIFeatures::convert_simple_real_to_sparse_real(CRealFe
 
 CStringFeatures<CHAR>* CGUIFeatures::convert_simple_char_to_string_char(CCharFeatures* src, CHAR* param)
 {
-	INT num_vec=src->get_num_vectors();
-	T_STRING<CHAR>* strings=new T_STRING<CHAR>[num_vec];
-	INT max_len=-1;
-
-	for (INT i=0; i<num_vec; i++)
+	if ( (src) && ( (src->get_feature_class()) == C_SIMPLE) )
 	{
-		bool to_free=false;
-		INT len=0;
-		CHAR* str= src->get_feature_vector(i, len, to_free);
-		strings[i].length=len ;
-		for (int j=0; j<len; j++)
-			if (str[j]==0)
-			{
-				strings[i].length=j ;
-				break ;
-			} ;
-		strings[i].string=new CHAR[strings[i].length];
+		INT num_vec=src->get_num_vectors();
+		T_STRING<CHAR>* strings=new T_STRING<CHAR>[num_vec];
+		INT max_len=-1;
 
-		for (int j=0; j<strings[i].length; j++)
-			strings[i].string[j]=str[j];
+		for (INT i=0; i<num_vec; i++)
+		{
+			bool to_free=false;
+			INT len=0;
+			CHAR* str= src->get_feature_vector(i, len, to_free);
+			strings[i].length=len ;
+			for (int j=0; j<len; j++)
+				if (str[j]==0)
+				{
+					strings[i].length=j ;
+					break ;
+				} ;
+			strings[i].string=new CHAR[strings[i].length];
 
-		if (strings[i].length> max_len)
-			max_len=strings[i].length;
+			for (int j=0; j<strings[i].length; j++)
+				strings[i].string[j]=str[j];
 
-		src->free_feature_vector(str, i, to_free);
+			if (strings[i].length> max_len)
+				max_len=strings[i].length;
+
+			src->free_feature_vector(str, i, to_free);
+		}
+
+		CStringFeatures<CHAR>* target= new CStringFeatures<CHAR>(new CAlphabet(src->get_alphabet()));
+		ASSERT(target);
+		target->set_features(strings, num_vec, max_len);
+		return target;
 	}
+	else
+		SG_ERROR( "no features of class/type SIMPLE/CHAR available\n");
 
-	CStringFeatures<CHAR>* target= new CStringFeatures<CHAR>(new CAlphabet(src->get_alphabet()));
-	target->set_features(strings, num_vec, max_len);
-
-	return target;
+	return NULL;
 }
 
 CRealFeatures* CGUIFeatures::convert_simple_word_to_simple_salzberg(CWordFeatures* src, CHAR* param)
