@@ -14,52 +14,50 @@
 #ifndef HAVE_SWIG
 #include "gui/GUI.h"
 #include "guilib/GUIKernel.h"
-#include "guilib/GUISVM.h"
 #include "guilib/GUIPluginEstimate.h"
 #include "kernel/Kernel.h"
 #include "kernel/CombinedKernel.h"
 #include "kernel/Chi2Kernel.h"
 #include "kernel/LinearKernel.h"
 #include "kernel/LinearByteKernel.h"
-#include "kernel/LinearCharKernel.h"
+#include "kernel/LinearStringKernel.h"
 #include "kernel/LinearWordKernel.h"
-#include "kernel/WeightedDegreeCharKernel.h"
-#include "kernel/WeightedDegreeCharKernelPolyA.h"
-#include "kernel/WeightedDegreePositionCharKernel.h"
-#include "kernel/WeightedDegreePositionPhylCharKernel.h"
-#include "kernel/FixedDegreeCharKernel.h"
-#include "kernel/LocalityImprovedCharKernel.h"
-#include "kernel/SimpleLocalityImprovedCharKernel.h"
+#include "kernel/WeightedDegreeStringKernel.h"
+#include "kernel/WeightedDegreePositionStringKernel.h"
+#include "kernel/FixedDegreeStringKernel.h"
+#include "kernel/LocalityImprovedStringKernel.h"
+#include "kernel/SimpleLocalityImprovedStringKernel.h"
 #include "kernel/PolyKernel.h"
-#include "kernel/CharPolyKernel.h"
 #include "kernel/CustomKernel.h"
 #include "kernel/ConstKernel.h"
 #include "kernel/PolyMatchWordKernel.h"
-#include "kernel/PolyMatchCharKernel.h"
+#include "kernel/PolyMatchStringKernel.h"
+#include "kernel/LocalAlignmentStringKernel.h"
 #include "kernel/WordMatchKernel.h"
-#include "kernel/CommWordKernel.h"
 #include "kernel/HammingWordKernel.h"
-#include "kernel/ManhattenWordKernel.h"
+#include "kernel/ManhattanWordKernel.h"
 #include "kernel/CanberraWordKernel.h"
 #include "kernel/CommWordStringKernel.h"
+#include "kernel/WeightedCommWordStringKernel.h"
 #include "kernel/CommUlongStringKernel.h"
 #include "kernel/HistogramWordKernel.h"
 #include "kernel/SalzbergWordKernel.h"
 #include "kernel/GaussianKernel.h"
+#include "kernel/GaussianShiftKernel.h"
 #include "kernel/SigmoidKernel.h"
 #include "kernel/SparseLinearKernel.h"
 #include "kernel/SparsePolyKernel.h"
 #include "kernel/SparseGaussianKernel.h"
-#include "kernel/SparseNormSquaredKernel.h"
 #include "kernel/DiagKernel.h"
 #include "kernel/MindyGramKernel.h"
 #include "kernel/DistanceKernel.h"
+#include "classifier/svm/SVM.h"
 #include "lib/io.h"
 #include "gui/GUI.h"
 
 #include <string.h>
 
-CGUIKernel::CGUIKernel(CGUI * gui_): gui(gui_)
+CGUIKernel::CGUIKernel(CGUI * gui_): CSGObject(), gui(gui_)
 {
 	kernel=NULL;
 	initialized=false;
@@ -160,20 +158,21 @@ bool CGUIKernel::init_kernel_optimization(CHAR* param)
 
 	kernel->set_precompute_matrix(false, false);
 
-	if (gui->guisvm.get_svm()!=NULL)
+	if (gui->guiclassifier.get_classifier()!=NULL)
 	{
+		CSVM* svm=(CSVM*) gui->guiclassifier.get_classifier();
 		if (kernel->has_property(KP_LINADD))
 		{
-			INT * sv_idx    = new INT[gui->guisvm.get_svm()->get_num_support_vectors()] ;
-			DREAL* sv_weight = new DREAL[gui->guisvm.get_svm()->get_num_support_vectors()] ;
+			INT * sv_idx    = new INT[svm->get_num_support_vectors()] ;
+			DREAL* sv_weight = new DREAL[svm->get_num_support_vectors()] ;
 			
-			for(INT i=0; i<gui->guisvm.get_svm()->get_num_support_vectors(); i++)
+			for(INT i=0; i<svm->get_num_support_vectors(); i++)
 			{
-				sv_idx[i]    = gui->guisvm.get_svm()->get_support_vector(i) ;
-				sv_weight[i] = gui->guisvm.get_svm()->get_alpha(i) ;
+				sv_idx[i]    = svm->get_support_vector(i) ;
+				sv_weight[i] = svm->get_alpha(i) ;
 			}
 
-			bool ret = kernel->init_optimization(gui->guisvm.get_svm()->get_num_support_vectors(), sv_idx, sv_weight) ;
+			bool ret = kernel->init_optimization(svm->get_num_support_vectors(), sv_idx, sv_weight) ;
 			
 			delete[] sv_idx ;
 			delete[] sv_weight ;
@@ -262,7 +261,7 @@ bool CGUIKernel::init_kernel(CHAR* param)
 					}
 					else
 					{
-						SG_INFO( "initialising kernel with TEST DATA, train: %d test %d\n",gui->guifeatures.get_train_features(), gui->guifeatures.get_test_features() );
+						SG_INFO( "initialising kernel with TEST DATA, train: %p test %p\n",gui->guifeatures.get_train_features(), gui->guifeatures.get_test_features() );
 						// lhs -> always train_features; rhs -> always test_features
 						kernel->init(gui->guifeatures.get_train_features(), gui->guifeatures.get_test_features());
 					} ;
@@ -380,9 +379,9 @@ CKernel* CGUIKernel::create_kernel(CHAR* param)
 				sscanf(param, "%s %s %d %le", kern_type, data_type, &size, &scale);
 				delete k;
 				if (scale==-1)
-					k=new CLinearWordKernel(size, true);
+					k=new CLinearByteKernel(size, true);
 				else
-					k=new CLinearWordKernel(size, false, scale);
+					k=new CLinearByteKernel(size, false, scale);
 				return k;
 			}
 			else if (strcmp(data_type,"WORD")==0)
@@ -402,31 +401,25 @@ CKernel* CGUIKernel::create_kernel(CHAR* param)
 				sscanf(param, "%s %s %d %le", kern_type, data_type, &size, &scale);
 				delete k;
 				if (scale==-1)
-					k=new CLinearCharKernel(size, true);
+					k=new CLinearStringKernel(size, true);
 				else
-					k=new CLinearCharKernel(size, false, scale);
+					k=new CLinearStringKernel(size, false, scale);
 				return k;
 			}
 			else if (strcmp(data_type,"REAL")==0)
 			{
-				double scale = -1 ;
+				double scale = 0;
 				sscanf(param, "%s %s %d %le", kern_type, data_type, &size, &scale);
 				delete k;
-				if (scale==-1)
-					k=new CLinearKernel(size, true);
-				else
-					k=new CLinearKernel(size, false, scale);
+				k=new CLinearKernel(size, scale);
 				return k;
 			}
 			else if (strcmp(data_type,"SPARSEREAL")==0)
 			{
-				double scale = -1 ;
+				double scale = 0;
 				sscanf(param, "%s %s %d %le", kern_type, data_type, &size, &scale);
 				delete k;
-				if (scale==-1)
-					k=new CSparseLinearKernel(size, true);
-				else
-					k=new CSparseLinearKernel(size, false, scale);
+				k=new CSparseLinearKernel(size, scale);
 				return k;
 			}
 		}
@@ -530,11 +523,11 @@ CKernel* CGUIKernel::create_kernel(CHAR* param)
 
 				sscanf(param, "%s %s %d %d %d %d", kern_type, data_type, &size, &degree, &inhomogene, &normalize);
 				delete k;
-				k=new CPolyMatchCharKernel(size, degree, inhomogene==1, normalize==1);
+				k=new CPolyMatchStringKernel(size, degree, inhomogene==1, normalize==1);
 
 				if (k)
 				{
-					SG_INFO( "PolyMatchCharKernel created\n");
+					SG_INFO( "PolyMatchStringKernel created\n");
 					return k;
 				}
 			}
@@ -555,7 +548,7 @@ CKernel* CGUIKernel::create_kernel(CHAR* param)
 				}
 			}
 		}
-		else if (strcmp(kern_type,"COMMSTRING")==0)
+		else if (strcmp(kern_type,"WEIGHTEDCOMMSTRING")==0 || strcmp(kern_type,"COMMSTRING")==0)
 		{
 			if (strcmp(data_type,"WORD")==0)
 			{
@@ -606,15 +599,31 @@ CKernel* CGUIKernel::create_kernel(CHAR* param)
 					return NULL ;
 				}
 
-				k=new CCommWordStringKernel(size, use_sign==1, normalization);
-				
-				if (k)
+				if (strcmp(kern_type,"WEIGHTEDCOMMSTRING")==0)
 				{
-					if (use_sign)
-						SG_INFO( "CommWordStringKernel with sign(count) created\n");
-					else
-						SG_INFO( "CommWordStringKernel with count created\n");
-					return k;
+					k=new CWeightedCommWordStringKernel(size, use_sign==1, normalization);
+
+					if (k)
+					{
+						if (use_sign)
+							SG_INFO( "WeightedCommWordStringKernel with sign(count) created\n");
+						else
+							SG_INFO( "WeightedCommWordStringKernel with count created\n");
+						return k;
+					}
+				}
+				else
+				{
+					k=new CCommWordStringKernel(size, use_sign==1, normalization);
+
+					if (k)
+					{
+						if (use_sign)
+							SG_INFO( "CommWordStringKernel with sign(count) created\n");
+						else
+							SG_INFO( "CommWordStringKernel with count created\n");
+						return k;
+					}
 				}
 			}
 			else if (strcmp(data_type,"ULONG")==0)
@@ -678,69 +687,6 @@ CKernel* CGUIKernel::create_kernel(CHAR* param)
 				}
 			}
 		}
-		else if (strcmp(kern_type,"COMM")==0)
-		{
-			if (strcmp(data_type,"WORD")==0)
-			{
-				INT use_sign = 0 ;
-				char normalization_str[100]="" ;
-				ENormalizationType normalization = FULL_NORMALIZATION ;
-				
-				sscanf(param, "%s %s %d %d %s", kern_type, data_type, &size, &use_sign, normalization_str);
-				if (strlen(normalization_str)==0)
-				{
-					normalization = FULL_NORMALIZATION ;
-					SG_INFO( "using full normalization\n") ;
-				}
-				else if (strcmp(normalization_str, "NO")==0)
-				{
-					normalization = NO_NORMALIZATION ;
-					SG_INFO( "using no normalization\n") ;
-				}
-				else if (strcmp(normalization_str, "SQRT")==0)
-				{
-					normalization = SQRT_NORMALIZATION ;
-					SG_INFO( "using sqrt normalization\n") ;
-				}
-				else if (strcmp(normalization_str, "SQRTLEN")==0)
-				{
-					normalization = SQRTLEN_NORMALIZATION ;
-					SG_INFO( "using sqrt-len normalization\n") ;
-				}
-				else if (strcmp(normalization_str, "LEN")==0)
-				{
-					normalization = LEN_NORMALIZATION ;
-					SG_INFO( "using len normalization\n") ;
-				}
-				else if (strcmp(normalization_str, "SQLEN")==0)
-				{
-					normalization = SQLEN_NORMALIZATION ;
-					SG_INFO( "using squared len normalization\n") ;
-				}
-				else if (strcmp(normalization_str, "FULL")==0)
-				{
-					normalization = FULL_NORMALIZATION ;
-					SG_INFO( "using full normalization\n") ;
-				}
-				else 
-				{
-					SG_ERROR( "unknow normalization: %s\n", normalization_str) ;
-					return NULL ;
-				}
-
-				delete k;
-				k=new CCommWordKernel(size, use_sign==1, normalization);
-
-				if (k)
-				{
-					if (use_sign)
-						SG_INFO( "CommWordKernel with sign(count) created\n");
-					else
-						SG_INFO( "CommWordKernel with count created\n");
-					return k;
-				}
-			}
-		}
 		else if (strcmp(kern_type,"HAMMING")==0)
 		{
 			if (strcmp(data_type,"WORD")==0)
@@ -770,11 +716,11 @@ CKernel* CGUIKernel::create_kernel(CHAR* param)
 				
 				sscanf(param, "%s %s %d %lf", kern_type, data_type, &size, &width);
 				delete k;
-				k=new CManhattenWordKernel(size, width);
+				k=new CManhattanWordKernel(size, width);
 
 				if (k)
 				{
-					SG_INFO( "ManhattenWordKernel created\n");
+					SG_INFO( "ManhattanWordKernel created\n");
 					return k;
 				}
 			}
@@ -800,9 +746,11 @@ CKernel* CGUIKernel::create_kernel(CHAR* param)
 		{
 			if (strcmp(data_type,"REAL")==0)
 			{
-				sscanf(param, "%s %s %d", kern_type, data_type, &size);
+				double width=1;
+
+				sscanf(param, "%s %s %d %lf", kern_type, data_type, &size, &width);
 				delete k;
-				k=new CChi2Kernel(size);
+				k=new CChi2Kernel(size, width);
 
 				if (k)
 				{
@@ -819,18 +767,33 @@ CKernel* CGUIKernel::create_kernel(CHAR* param)
 
 				sscanf(param, "%s %s %d %d", kern_type, data_type, &size, &d);
 				delete k;
-				k=new CFixedDegreeCharKernel(size, d);
+				k=new CFixedDegreeStringKernel(size, d);
 
 				if (k)
 				{
-					SG_INFO( "FixedDegreeCharKernel created\n");
+					SG_INFO( "FixedDegreeStringKernel created\n");
 					return k;
 				}
 			}
 		}
-		else if ((strcmp(kern_type,"WEIGHTEDDEGREEPOS2")==0) || (strcmp(kern_type,"WEIGHTEDDEGREEPOSPHYL2")==0))
+		else if (strcmp(kern_type,"LOCALALIGNMENT")==0)
 		{
 			if (strcmp(data_type,"CHAR")==0)
+			{
+				sscanf(param, "%s %s %d", kern_type, data_type, &size);
+				delete k;
+				k=new CLocalAlignmentStringKernel(size);
+
+				if (k)
+				{
+					SG_INFO( "LocalAlignmentStringKernel created\n");
+					return k;
+				}
+			}
+		}
+		else if (strcmp(kern_type,"WEIGHTEDDEGREEPOS2")==0)
+		{
+			if ((strcmp(data_type,"CHAR")==0) || (strcmp(data_type,"STRING")==0))
 			{
 				INT d=3;
 				INT max_mismatch = 0 ;
@@ -859,8 +822,6 @@ CKernel* CGUIKernel::create_kernel(CHAR* param)
 					} ;
 					strcpy(rest,rest2) ;
 				}
-				//for (INT i=0; i<length; i++)
-				//  SG_INFO( "shift[%i]=%i\n", i, shift[i]) ;
 				
 				DREAL* weights=new DREAL[d*(1+max_mismatch)];
 				DREAL sum=0;
@@ -886,32 +847,97 @@ CKernel* CGUIKernel::create_kernel(CHAR* param)
 							weights[i+j*d]= 0;
 					} ;
 				} ;
-				
-				
+
+
 				delete k;
 
-				if (strcmp(kern_type,"WEIGHTEDDEGREEPOS2")==0)
-					k=new CWeightedDegreePositionCharKernel(size, weights, 
-															d, max_mismatch, 
-															shift, length, true);
-				else
-					k=new CWeightedDegreePositionPhylCharKernel(size, weights, 
-															d, max_mismatch, 
-															shift, length, true);
-
+				k=new CWeightedDegreePositionStringKernel(size, weights,
+					d, max_mismatch, shift, length, true);
 				delete[] shift ;
 				delete[] weights ;
-				
+
 				if (k)
 				{
-					SG_INFO( "WeightedDegreePositionCharKernel(%d,.,%d,%d,.,%d) created\n",size, d, max_mismatch, length);
+					SG_INFO( "WeightedDegreePositionStringKernel(%d,.,%d,%d,.,%d) created\n",size, d, max_mismatch, length);
 					return k;
 				}
 			}
 		}
-		else if ((strcmp(kern_type,"WEIGHTEDDEGREEPOS3")==0) || (strcmp(kern_type,"WEIGHTEDDEGREEPOSPHYL3")==0))
+		else if (strcmp(kern_type,"WEIGHTEDDEGREEPOS2_NONORM")==0)
 		{
-			if (strcmp(data_type,"CHAR")==0)
+			if ((strcmp(data_type,"CHAR")==0) || (strcmp(data_type,"STRING")==0))
+			{
+				INT d=3;
+				INT max_mismatch = 0 ;
+				INT i=0;
+				INT length = 0 ;
+				char * rest = new char[strlen(param)] ;
+				char * rest2 = new char[strlen(param)] ;
+				
+				sscanf(param, "%s %s %d %d %d %d %[0-9 .+-]", 
+					   kern_type, data_type, &size, &d, &max_mismatch, 
+					   &length, rest);
+
+				INT *shift = new INT[length] ;
+				for (i=0; i<length; i++)
+				{
+					int args = sscanf(rest, "%i %[0-9 .+-]", &shift[i], rest2) ;
+					if (((args!=2) && (i<length-1)) || (args<1))
+					{
+						SG_ERROR( "failed to read list at position %i\n", i) ;
+						return false ;
+					} ;
+					if (shift[i]>length)
+					{
+						SG_ERROR( "shift longer than sequence: %i \n", shift[i]) ;
+						return false ;
+					} ;
+					strcpy(rest,rest2) ;
+				}
+				
+				DREAL* weights=new DREAL[d*(1+max_mismatch)];
+				DREAL sum=0;
+
+				for (i=0; i<d; i++)
+				{
+					weights[i]=d-i;
+					sum+=weights[i];
+				}
+				for (i=0; i<d; i++)
+					weights[i]/=sum;
+				
+				for (i=0; i<d; i++)
+				{
+					for (INT j=1; j<=max_mismatch; j++)
+					{
+						if (j<i+1)
+						{
+							INT nk=CMath::nchoosek(i+1, j) ;
+							weights[i+j*d]=weights[i]/(nk*pow(3,j)) ;
+						}
+						else
+							weights[i+j*d]= 0;
+					} ;
+				} ;
+
+
+				delete k;
+
+				k=new CWeightedDegreePositionStringKernel(size, weights,
+					d, max_mismatch, shift, length, false);
+				delete[] shift ;
+				delete[] weights ;
+
+				if (k)
+				{
+					SG_INFO( "WeightedDegreePositionStringKernel(%d,.,%d,%d,.,%d) w/o NORMALIZATION created\n",size, d, max_mismatch, length);
+					return k;
+				}
+			}
+		}
+		else if (strcmp(kern_type,"WEIGHTEDDEGREEPOS3")==0)
+		{
+			if ((strcmp(data_type,"CHAR")==0) || (strcmp(data_type,"STRING")==0))
 			{
 				INT d=3;
 				INT max_mismatch = 0 ;
@@ -999,34 +1025,26 @@ CKernel* CGUIKernel::create_kernel(CHAR* param)
 				
 				
 				delete k;
-				
-				if (strcmp(kern_type,"WEIGHTEDDEGREEPOS3")==0)
-					k=new CWeightedDegreePositionCharKernel(size, weights, 
-															d, max_mismatch, 
-															shift, length, false, 
-															mkl_stepsize);
-				else
-					k=new CWeightedDegreePositionPhylCharKernel(size, weights, 
-																d, max_mismatch, 
-																shift, length, false, 
-																mkl_stepsize);
-				
-				((CWeightedDegreePositionCharKernel*)k)->set_position_weights(position_weights, length) ;
-				
+
+				k=new CWeightedDegreePositionStringKernel(size,
+					weights, d, max_mismatch, shift, length,
+					false, mkl_stepsize);
+				((CWeightedDegreePositionStringKernel*)k)->set_position_weights(position_weights, length);
+
 				delete[] shift ;
 				delete[] weights ;
 				delete[] position_weights ;
 				
 				if (k)
 				{
-					SG_INFO( "WeightedDegreePositionCharKernel(%d,.,%d,%d,.,%d,%d) created\n",size, d, max_mismatch, length,mkl_stepsize);
+					SG_INFO( "WeightedDegreePositionStringKernel(%d,.,%d,%d,.,%d,%d) created\n",size, d, max_mismatch, length,mkl_stepsize);
 					return k;
 				}
 			}
 		}
-		else if ((strcmp(kern_type,"WEIGHTEDDEGREEPOS")==0) || (strcmp(kern_type,"WEIGHTEDDEGREEPOSPHYL")==0))
+		else if (strcmp(kern_type,"WEIGHTEDDEGREEPOS")==0)
 		{
-			if (strcmp(data_type,"CHAR")==0)
+			if ((strcmp(data_type,"CHAR")==0) || (strcmp(data_type,"STRING")==0))
 			{
 				INT d=3;
 				INT max_mismatch = 0 ;
@@ -1081,27 +1099,21 @@ CKernel* CGUIKernel::create_kernel(CHAR* param)
 				
 				delete k;
 
-				if (strcmp(kern_type,"WEIGHTEDDEGREEPOS")==0)
-					k=new CWeightedDegreePositionCharKernel(size, weights, 
-															d, max_mismatch, 
-															shift, length);
-				else
-					k=new CWeightedDegreePositionCharKernel(size, weights, 
-															d, max_mismatch, 
-															shift, length);
+				k=new CWeightedDegreePositionStringKernel(size, weights, d, max_mismatch, shift, length);
+
 				delete[] shift ;
 				delete[] weights ;
 				
 				if (k)
 				{
-					SG_INFO( "WeightedDegreePositionCharKernel(%d,.,%d,%d,.,%d) created\n",size, d, max_mismatch, length);
+					SG_INFO( "WeightedDegreePositionStringKernel(%d,.,%d,%d,.,%d) created\n",size, d, max_mismatch, length);
 					return k;
 				}
 			}
 		}
 		else if (strcmp(kern_type,"WEIGHTEDDEGREE")==0)
 		{
-			if (strcmp(data_type,"CHAR")==0)
+			if ((strcmp(data_type,"CHAR")==0) || (strcmp(data_type,"STRING")==0))
 			{
 				INT use_normalization=1;
 				INT d=3;
@@ -1151,58 +1163,14 @@ CKernel* CGUIKernel::create_kernel(CHAR* param)
 				}
 
 				delete k;
-				k=new CWeightedDegreeCharKernel(size, weights, d, max_mismatch, use_normalization==1, block_computation==1, mkl_stepsize);
+				k=new CWeightedDegreeStringKernel(size, weights,
+					d, max_mismatch, use_normalization==1,
+					block_computation==1, mkl_stepsize);
 				delete[] weights ;
 				
 				if (k)
 				{
-					SG_INFO( "WeightedDegreeCharKernel created\n");
-					return k;
-				}
-			}
-		}
-		else if (strcmp(kern_type,"WEIGHTEDDEGREEPOLYA")==0)
-		{
-			if (strcmp(data_type,"CHAR")==0)
-			{
-				INT d=3;
-				INT max_mismatch = 0;
-				INT i=0;
-
-				sscanf(param, "%s %s %d %d %d", kern_type, data_type, &size, &d, &max_mismatch);
-				DREAL* weights=new DREAL[d*(1+max_mismatch)];
-				DREAL sum=0;
-
-				for (i=0; i<d; i++)
-				{
-					weights[i]=d-i;
-					sum+=weights[i];
-				}
-				for (i=0; i<d; i++)
-					weights[i]/=sum;
-				
-				for (i=0; i<d; i++)
-				{
-					for (INT j=1; j<=max_mismatch; j++)
-					{
-						if (j<i+1)
-						{
-							INT nk=CMath::nchoosek(i+1, j);
-							weights[i+j*d]=weights[i]/(nk*pow(3,j));
-						}
-						else
-							weights[i+j*d]= 0;
-						
-					}
-				}
-				
-				delete k;
-				k=new CWeightedDegreeCharKernelPolyA(size, weights, d, max_mismatch);
-				delete[] weights ;
-				
-				if (k)
-				{
-					SG_INFO( "WeightedDegreeCharKernelPolyA created\n");
+					SG_INFO( "WeightedDegreeStringKernel created\n");
 					return k;
 				}
 			}
@@ -1216,10 +1184,10 @@ CKernel* CGUIKernel::create_kernel(CHAR* param)
 				INT d2=1;
 				sscanf(param, "%s %s %d %d %d %d", kern_type, data_type, &size, &l, &d1, &d2);
 				delete k;
-				k=new CSimpleLocalityImprovedCharKernel(size, l, d1, d2);
+				k=new CSimpleLocalityImprovedStringKernel(size, l, d1, d2);
 				if (k)
 				{
-					SG_INFO( "SimpleLocalityImprovedCharKernel created\n");
+					SG_INFO( "SimpleLocalityImprovedStringKernel created\n");
 					return k;
 				}
 			}
@@ -1233,10 +1201,10 @@ CKernel* CGUIKernel::create_kernel(CHAR* param)
 				INT d2=1;
 				sscanf(param, "%s %s %d %d %d %d", kern_type, data_type, &size, &l, &d1, &d2);
 				delete k;
-				k=new CLocalityImprovedCharKernel(size, l, d1, d2);
+				k=new CLocalityImprovedStringKernel(size, l, d1, d2);
 				if (k)
 				{
-					SG_INFO( "LocalityImprovedCharKernel created\n");
+					SG_INFO( "LocalityImprovedStringKernel created\n");
 					return k;
 				}
 			}
@@ -1259,22 +1227,6 @@ CKernel* CGUIKernel::create_kernel(CHAR* param)
 					return k;
 				}
 			}
-			else if (strcmp(data_type,"CHAR")==0)
-			{
-				INT inhomogene=0;
-				INT degree=2;
-				INT normalize=1;
-
-				sscanf(param, "%s %s %d %d %d %d", kern_type, data_type, &size, &degree, &inhomogene, &normalize);
-				delete k;
-				k=new CCharPolyKernel(size, degree, inhomogene==1, normalize==1);
-
-				if (k)
-				{
-					SG_INFO( "CharPolynomial Kernel created\n");
-					return k;
-				}
-			}
 			else if (strcmp(data_type,"SPARSEREAL")==0)
 			{
 				INT inhomogene=0;
@@ -1338,16 +1290,20 @@ CKernel* CGUIKernel::create_kernel(CHAR* param)
 				}
 			}
 		}
-		else if (strcmp(kern_type,"NORMSQUARED")==0)
+		else if (strcmp(kern_type,"GAUSSIANSHIFT")==0) // RBF
 		{
-			if (strcmp(data_type,"SPARSEREAL")==0)
+			if (strcmp(data_type,"REAL")==0)
 			{
-				sscanf(param, "%s %s %d", kern_type, data_type, &size);
+				double width=1;
+				INT max_shift=0 ;
+				INT shift_step=1 ;
+
+				sscanf(param, "%s %s %d %lf %i %i", kern_type, data_type, &size, &width, &max_shift, &shift_step);
 				delete k;
-				k=new CSparseNormSquaredKernel(size);
+				k=new CGaussianShiftKernel(size, width, max_shift, shift_step);
 				if (k)
 				{
-					SG_INFO( "Sparse NormSquared Kernel created\n");
+					SG_INFO( "GaussianShift Kernel created\n");
 					return k;
 				}
 			}
