@@ -30,13 +30,13 @@ using namespace std;
 #include "features/Labels.h"
 #include "features/RealFeatures.h"
 #include "features/CharFeatures.h"
-#include "kernel/WeightedDegreeCharKernel.h"
+#include "kernel/WeightedDegreeStringKernel.h"
 #include "kernel/CommWordStringKernel.h"
 #include "classifier/svm/SVM.h"
 
 extern CTextGUI* gui;
 
-CGUIOctave::CGUIOctave()
+CGUIOctave::CGUIOctave() : CSGObject()
 {
 }
 
@@ -331,10 +331,11 @@ bool CGUIOctave::one_class_hmm_classify_example(octave_value_list& retvals, int
 
 bool CGUIOctave::get_svm(octave_value_list& retvals)
 {
-	CSVM* svm=gui->guisvm.get_svm();
+	CSVM* svm=(CSVM*) gui->guiclassifier.get_classifier();
 
 	if (svm)
 	{
+		SG_DEBUG("num alphas: %d\n", svm->get_num_support_vectors());
 		Matrix alphas=Matrix(svm->get_num_support_vectors(), 2);
 		double b=0;
 
@@ -345,8 +346,8 @@ bool CGUIOctave::get_svm(octave_value_list& retvals)
 
 			for (int i=0; i< svm->get_num_support_vectors(); i++)
 			{
-				alphas(1,i)=svm->get_alpha(i);
-				alphas(2,i)=svm->get_support_vector(i);
+				alphas(i,0)= (double) svm->get_alpha(i);
+				alphas(i,1)= (double) svm->get_support_vector(i);
 			}
 
 			retvals(0)=b;
@@ -361,7 +362,7 @@ bool CGUIOctave::get_svm(octave_value_list& retvals)
 
 bool CGUIOctave::set_svm(const octave_value_list& vals)
 {
-	CSVM* svm=gui->guisvm.get_svm();
+	CSVM* svm=(CSVM*) gui->guiclassifier.get_classifier();
 
 	if (svm)
 	{
@@ -375,8 +376,8 @@ bool CGUIOctave::set_svm(const octave_value_list& vals)
 
 			for (int i=0; i< svm->get_num_support_vectors(); i++)
 			{
-				svm->set_alpha(i, alphas(1, i));
-				svm->set_support_vector(i, (int) alphas(2,i));
+				svm->set_alpha(i, alphas(i,0));
+				svm->set_support_vector(i, (int) alphas(i,1));
 			}
 
 			return true;
@@ -393,7 +394,7 @@ bool CGUIOctave::svm_classify(octave_value_list& retvals)
 	{
 		int num_vec=f->get_num_vectors();
 
-		CLabels* l=gui->guisvm.classify();
+		CLabels* l=gui->guiclassifier.classify();
 
 		if (!l)
 		{
@@ -412,11 +413,37 @@ bool CGUIOctave::svm_classify(octave_value_list& retvals)
 	return false;
 }
 
-bool CGUIOctave::svm_classify_example(octave_value_list& retvals, int idx)
+bool CGUIOctave::classify(octave_value_list& retvals)
+{
+	CFeatures* f=gui->guifeatures.get_test_features();
+	if (f)
+	{
+		int num_vec=f->get_num_vectors();
+
+		CLabels* l=gui->guiclassifier.classify();
+
+		if (!l)
+		{
+			SG_ERROR( "classify failed\n") ;
+			return false ;
+		} ;
+
+		RowVector result=RowVector(num_vec);
+		for (int i=0; i<num_vec; i++)
+			result(i)=l->get_label(i);
+		delete l;
+
+		retvals(0)=result;
+		return true;
+	}
+	return false;
+}
+
+bool CGUIOctave::classify_example(octave_value_list& retvals, int idx)
 {
 	double result=0;
 
-	if (!gui->guisvm.classify_example(idx, result))
+	if (!gui->guiclassifier.classify_example(idx, result))
 	{
 		SG_ERROR( "svm_classify_example failed\n") ;
 		return false ;
@@ -566,7 +593,7 @@ bool CGUIOctave::get_features(octave_value_list& retvals, CFeatures* f)
 			return (!value.is_empty());
 		}
 		else
-			SG_ERROR( "matlab does not support that feature type\n");
+			SG_ERROR( "octave does not support that feature type\n");
 
 	}
 
@@ -654,29 +681,37 @@ CFeatures* CGUIOctave::set_features(const octave_value_list& vals)
 				{
 					CHAR* al=CGUIOctave::get_octaveString(vals(3).string_value());
 					CAlphabet* alpha= new CAlphabet(al, strlen(al));
+					ASSERT(alpha);
 					
 					INT num_vec = cm.cols();
 					INT num_feat = cm.rows();
+					T_STRING<CHAR>* sc=new T_STRING<CHAR>[num_vec];
+					ASSERT(sc);
 					
-					CHAR* fm=new CHAR[num_vec*num_feat];
-					ASSERT(fm);
+					int maxlen=num_feat;
+					alpha->clear_histogram();
+
+					for (int i=0; i<num_vec; i++)
+					{
+						sc[i].length=num_feat;
+						sc[i].string=new CHAR[num_feat];
+						ASSERT(sc[i].string)
 
-					for (INT i=0; i<num_vec; i++)
 						for (INT j=0; j<num_feat; j++)
-							fm[i*num_feat+j]= (CHAR) cm(j,i);
+							sc[i].string[j]=(CHAR) cm(j,i);
 
-					alpha->add_string_to_histogram(fm, ((LONG) num_vec)* ((LONG) num_feat));
-                    SG_INFO("max_value_in_histogram:%d\n", alpha->get_max_value_in_histogram());
-                    SG_INFO("num_symbols_in_histogram:%d\n", alpha->get_num_symbols_in_histogram());
-
-					f= new CCharFeatures(alpha, 0);
+						alpha->add_string_to_histogram(sc[i].string, sc[i].length);
+					}
+					SG_INFO("max_value_in_histogram:%d\n", alpha->get_max_value_in_histogram());
+					SG_INFO("num_symbols_in_histogram:%d\n", alpha->get_num_symbols_in_histogram());
+					f= new CStringFeatures<CHAR>(alpha);
 					ASSERT(f);
 
 					if (alpha->check_alphabet_size() && alpha->check_alphabet())
-						((CCharFeatures*) f)->set_feature_matrix(fm, num_feat, num_vec);
+						((CStringFeatures<CHAR>*) f)->set_features(sc, num_vec, maxlen);
 					else
 					{
-						((CCharFeatures*) f)->set_feature_matrix(fm, num_feat, num_vec);
+						((CStringFeatures<CHAR>*) f)->set_features(sc, num_vec, maxlen);
 						delete f;
 						f=NULL;
 					}
@@ -787,7 +822,7 @@ bool CGUIOctave::get_kernel_optimization(octave_value_list& retvals)
 
 	if (kernel_ && (kernel_->get_kernel_type() == K_WEIGHTEDDEGREE))
 	{
-		CWeightedDegreeCharKernel *kernel = (CWeightedDegreeCharKernel *) kernel_ ;
+		CWeightedDegreeStringKernel *kernel = (CWeightedDegreeStringKernel *) kernel_ ;
 
 		if (kernel->get_max_mismatch()!=0)
 			return false;
