@@ -18,13 +18,15 @@
 #include "features/Features.h"
 #include "features/RealFeatures.h"
 #include "features/CharFeatures.h"
-#include "kernel/WeightedDegreeCharKernel.h"
-#include "kernel/WeightedDegreePositionCharKernel.h"
+#include "kernel/WeightedDegreeStringKernel.h"
+#include "kernel/WeightedDegreePositionStringKernel.h"
 #include "kernel/CombinedKernel.h"
 #include "kernel/CustomKernel.h"
+#include "classifier/svm/SVM.h"
+
+#include "lib/python.h"
 
 extern "C" {
-#include <Python.h>
 #include <object.h>
 #include <../../numarray/numpy/libnumarray.h>
 #include <numpy/ndarrayobject.h>
@@ -32,7 +34,7 @@ extern "C" {
 
 extern CTextGUI* gui;
 
-CGUIPython::CGUIPython()
+CGUIPython::CGUIPython() : CSGObject()
 {
 	import_libnumarray();
 }
@@ -45,7 +47,7 @@ PyObject* CGUIPython::py_send_command(PyObject* self, PyObject* args)
 {
 	char *cmd;
 
-	if (!PyArg_ParseTuple(args, "s", &cmd))
+	if (!PyArg_ParseTuple(args, (char*) "s", &cmd))
 		return NULL;
 
 	gui->parse_line(cmd);
@@ -58,7 +60,7 @@ PyObject* CGUIPython::py_system(PyObject* self, PyObject* args)
 {
 	char *cmd;
 
-	if (!PyArg_ParseTuple(args, "s", &cmd))
+	if (!PyArg_ParseTuple(args, (char*) "s", &cmd))
 		return NULL;
 	::system(cmd);
 
@@ -68,7 +70,7 @@ PyObject* CGUIPython::py_system(PyObject* self, PyObject* args)
 
 PyObject* CGUIPython::py_help(PyObject* self, PyObject* args)
 {
-	gui->parse_line("help");
+	gui->parse_line((char*) "help");
 
 	Py_INCREF(Py_None);
 	return Py_None;
@@ -96,12 +98,12 @@ PyObject* CGUIPython::py_get_viterbi(PyObject* self, PyObject* args)
 
 PyObject* CGUIPython::py_get_svm(PyObject* self, PyObject* args)
 {
-	CSVM* svm=gui->guisvm.get_svm();
+	CSVM* svm=(CSVM*) gui->guiclassifier.get_classifier();
 
 	if (svm && svm->get_num_support_vectors()>0)
 	{
 		PyArrayObject* py_alphas=NA_NewArray(NULL, tFloat64, 1, svm->get_num_support_vectors());
-		PyArrayObject* py_sv_idx=NA_NewArray(NULL, tInt32, 1, svm->get_num_support_vectors());
+		PyArrayObject* py_sv_idx=NA_NewArray(NULL, tInt64, 1, svm->get_num_support_vectors());
 		Float64 b=svm->get_bias();
 
 		SG_DEBUG("num_sv: %d\n", svm->get_num_support_vectors());
@@ -115,9 +117,9 @@ PyObject* CGUIPython::py_get_svm(PyObject* self, PyObject* args)
 			}
 
 			PyObject* ret=PyDict_New();
-			PyDict_SetItemString(ret, "b", Py_BuildValue("f",b));
-			PyDict_SetItemString(ret, "sv_idx", (PyObject*) py_sv_idx);
-			PyDict_SetItemString(ret, "alpha", (PyObject*) py_alphas);
+			PyDict_SetItemString(ret, (char*) "b", Py_BuildValue((char*) "f",b));
+			PyDict_SetItemString(ret, (char*) "sv_idx", (PyObject*) py_sv_idx);
+			PyDict_SetItemString(ret, (char*) "alpha", (PyObject*) py_alphas);
 
 			return ret;
 		}
@@ -134,13 +136,25 @@ PyObject* CGUIPython::py_get_kernel_matrix(PyObject* self, PyObject* args)
 
 	CKernel* k = gui->guikernel.get_kernel();
 
-	if (!k)
+	if (!k || !k->get_lhs() || !k->get_rhs())
 		SG_ERROR( "no kernel set\n");
 	else
-		result=k->get_kernel_matrix_real(m,n,NULL);
+	{
+		//result=k->get_kernel_matrix_real(m,n,NULL);
+		
+		m = k->get_lhs()->get_num_vectors();	
+		n = k->get_rhs()->get_num_vectors();
+		result = new DREAL[m*n];
+		ASSERT(result);
+
+		for (INT i=0; i<m; i++)
+			for (INT j=0; j<n; j++)
+				result[i*n+j]=k->kernel(i,j);
+		
+	}
 
 	if(result)
-		return (PyObject*) NA_NewArray(result, tFloat64, 2, m, n);
+		return (PyObject*) NA_NewArray(result, tFloat64, 2, m,n);
 	else
 		return NULL;
 }
@@ -160,7 +174,7 @@ PyObject* CGUIPython::py_set_subkernels_weights(PyObject* self, PyObject* args)
 {
 	PyObject* py_oweights = NULL;
 
-	if (PyArg_ParseTuple(args, "O", &py_oweights))
+	if (PyArg_ParseTuple(args, (char*) "O", &py_oweights))
 	{
 		PyArrayObject* py_weights = NA_InputArray(py_oweights, tFloat64, NUM_C_ARRAY);
 		CKernel* k= gui->guikernel.get_kernel();
@@ -171,7 +185,7 @@ PyObject* CGUIPython::py_set_subkernels_weights(PyObject* self, PyObject* args)
 
 			if (k->get_kernel_type() == K_WEIGHTEDDEGREE)
 			{
-				CWeightedDegreeCharKernel *kernel = (CWeightedDegreeCharKernel *) k;
+				CWeightedDegreeStringKernel *kernel = (CWeightedDegreeStringKernel *) k;
 				INT degree = kernel->get_degree() ;
 
 				if ((py_weights->nd == 1 && py_weights->dimensions[0] == degree) ||
@@ -191,7 +205,7 @@ PyObject* CGUIPython::py_set_subkernels_weights(PyObject* self, PyObject* args)
 			}
 			else if (k->get_kernel_type() == K_WEIGHTEDDEGREEPOS)
 			{
-				CWeightedDegreePositionCharKernel *kernel = (CWeightedDegreePositionCharKernel *) k;
+				CWeightedDegreePositionStringKernel *kernel = (CWeightedDegreePositionStringKernel *) k;
 				INT degree = kernel->get_degree() ;
 
 				if ((py_weights->nd == 1 && py_weights->dimensions[0] == degree) ||
@@ -249,7 +263,7 @@ PyObject* CGUIPython::py_get_subkernel_weights(PyObject* self, PyObject* args)
 	{
 		if (k->get_kernel_type() == K_WEIGHTEDDEGREE)
 		{
-			CWeightedDegreeCharKernel *kernel = (CWeightedDegreeCharKernel *) k;
+			CWeightedDegreeStringKernel *kernel = (CWeightedDegreeStringKernel *) k;
 
 			const DREAL* weights = kernel->get_degree_weights(degree, length) ;
 			if (length == 0)
@@ -265,7 +279,7 @@ PyObject* CGUIPython::py_get_subkernel_weights(PyObject* self, PyObject* args)
 		}
 		else if (k->get_kernel_type() == K_WEIGHTEDDEGREEPOS)
 		{
-			CWeightedDegreePositionCharKernel *kernel = (CWeightedDegreePositionCharKernel *) k;
+			CWeightedDegreePositionStringKernel *kernel = (CWeightedDegreePositionStringKernel *) k;
 
 			const DREAL* weights = kernel->get_degree_weights(degree, length) ;
 			if (length == 0)
@@ -310,7 +324,7 @@ PyObject* CGUIPython::py_get_features(PyObject* self, PyObject* args)
 {
 	char* target = NULL;
 
-	if (PyArg_ParseTuple(args, "s", &target))
+	if (PyArg_ParseTuple(args, (char*) "s", &target))
 	{
 		CFeatures* f = NULL;
 
@@ -377,7 +391,7 @@ PyObject* CGUIPython::py_get_labels(PyObject* self, PyObject* args)
 {
 	char* target = NULL;
 
-	if (PyArg_ParseTuple(args, "s", &target))
+	if (PyArg_ParseTuple(args, (char*) "s", &target))
 	{
 		CLabels* labels = NULL;
 
@@ -452,24 +466,24 @@ PyObject* CGUIPython::py_append_hmm(PyObject* self, PyObject* args)
 
 PyObject* CGUIPython::py_set_svm(PyObject* self, PyObject* args)
 {
-	CSVM* svm=gui->guisvm.get_svm();
+	CSVM* svm=(CSVM*) gui->guiclassifier.get_classifier();
 
 	if (svm)
 	{
 		PyObject* py_dict = NULL;
-		if (!PyArg_ParseTuple(args, "O", &py_dict))
+		if (!PyArg_ParseTuple(args, (char*) "O", &py_dict))
 			return NULL;
 		if (!PyDict_Check(py_dict))
 			return NULL;
 
-		PyObject* py_oalphas = PyDict_GetItemString(py_dict, "alphas");
+		PyObject* py_oalphas = PyDict_GetItemString(py_dict, "alpha");
 		PyObject* py_osv_idx = PyDict_GetItemString(py_dict, "sv_idx");
 		PyObject* py_bias = PyDict_GetItemString(py_dict, "b");
 
 		if (py_oalphas && py_osv_idx && py_bias)
 		{
 			PyArrayObject* py_alphas = NA_InputArray(py_oalphas, tFloat64, NUM_C_ARRAY);
-			PyArrayObject* py_sv_idx = NA_InputArray(py_oalphas, tInt64, NUM_C_ARRAY);
+			PyArrayObject* py_sv_idx = NA_InputArray(py_osv_idx, tInt64, NUM_C_ARRAY);
 
 			if (py_alphas && py_sv_idx && py_bias && (py_alphas->nd == 1) && (py_alphas->dimensions[0] > 0) && NA_ShapeEqual(py_alphas, py_sv_idx))
 			{
@@ -515,7 +529,7 @@ PyObject* CGUIPython::py_set_custom_kernel(PyObject* self, PyObject* args)
 	bool dest_is_diag = false;
 
 
-	if (PyArg_ParseTuple(args, "Os", &py_okernel, target))
+	if (PyArg_ParseTuple(args, (char*) "Os", &py_okernel, target))
 	{
 		if ( (!strncmp(target, "DIAG", strlen("DIAG"))) || 
 				(!strncmp(target, "FULL", strlen("FULL"))) ) 
@@ -600,8 +614,8 @@ PyObject* CGUIPython::py_set_features(PyObject* self, PyObject* args)
 	char* target = NULL;
 	char* cmdline = NULL;
 
-	if ( (PyArg_ParseTuple(args, "sO", &target, &py_ofeat)) ||
-			(PyArg_ParseTuple(args, "sOs", &target, &py_ofeat, &cmdline)) )
+	if ( (PyArg_ParseTuple(args, (char*) "sO", &target, &py_ofeat)) ||
+			(PyArg_ParseTuple(args, (char*) "sOs", &target, &py_ofeat, &cmdline)) )
 	{
 		if ( (!strncmp(target, "TRAIN", strlen("TRAIN"))) || 
 				(!strncmp(target, "TEST", strlen("TEST"))) ) 
@@ -638,8 +652,8 @@ PyObject* CGUIPython::py_add_features(PyObject* self, PyObject* args)
 	char* target = NULL;
 	char* cmdline = NULL;
 
-	if (PyArg_ParseTuple(args, "sOs", &target, &py_ofeat, &cmdline) ||
-            PyArg_ParseTuple(args, "sO", &target, &py_ofeat))
+	if (PyArg_ParseTuple(args, (char*) "sOs", &target, &py_ofeat, &cmdline) ||
+            PyArg_ParseTuple(args, (char*) "sO", &target, &py_ofeat))
 	{
 		if ( (!strncmp(target, "TRAIN", strlen("TRAIN"))) || 
 				(!strncmp(target, "TEST", strlen("TEST"))) ) 
@@ -675,7 +689,7 @@ PyObject* CGUIPython::py_set_labels(PyObject* self, PyObject* args)
 	PyObject *arg = NULL;
 	char* target = NULL;
 
-	if (PyArg_ParseTuple(args, "sO", &target, &arg))
+	if (PyArg_ParseTuple(args, (char*) "sO", &target, &arg))
 	{
 		if ( (!strncmp(target, "TRAIN", strlen("TRAIN"))) || 
 				(!strncmp(target, "TEST", strlen("TEST"))) ) 
@@ -746,7 +760,7 @@ PyObject* CGUIPython::py_svm_classify(PyObject* self, PyObject* args)
 	{
 		int num_vec=f->get_num_vectors();
 
-		CLabels* l=gui->guisvm.classify();
+		CLabels* l=gui->guiclassifier.classify();
 
 		if (l)
 		{
@@ -765,10 +779,10 @@ PyObject* CGUIPython::py_svm_classify_example(PyObject* self, PyObject* args)
 {
 	int idx = 0;
 
-	if (PyArg_ParseTuple(args, "i", &idx))
+	if (PyArg_ParseTuple(args, (char*) "i", &idx))
 	{
 		DREAL result;
-		if (!gui->guisvm.classify_example(idx, result))
+		if (!gui->guiclassifier.classify_example(idx, result))
 			SG_ERROR( "svm_classify_example failed\n") ;
 		else
 			return PyFloat_FromDouble(result);
@@ -802,7 +816,7 @@ PyObject* CGUIPython::py_test(PyObject* self, PyObject* args)
 	PyObject   *py_ofeat = NULL;
 	PyArrayObject *py_afeat = NULL;
 
-	if (PyArg_ParseTuple(args, "O", &py_ofeat))
+	if (PyArg_ParseTuple(args, (char*) "O", &py_ofeat))
 	{
 		/* Align, Byteswap, Contiguous, Typeconvert */
 		py_afeat  = NA_InputArray(py_ofeat, tFloat64, NUM_C_ARRAY);
@@ -863,27 +877,26 @@ CFeatures* CGUIPython::set_features(PyObject* arg, char* args)
 	}
 
 	typeno=PyArray(arg)->descr->type_num;
-
 	//strangely this seems to be a CharacterArray...
 	if (NA_NDArrayCheck(arg) && PyArray(arg)->descr->type_num == PyArray_NOTYPE)
 		typeno=tUInt8;
+	//SG_DEBUG("%d (%d) vs. %d : tInt8(%d),tUInt8(%d),tUInt32(%d),tInt32(%d)\n", typeno, PyArray_NOTYPE, NA_NumarrayType(arg), tInt8, tUInt8,tUInt32,tInt32);
 
-	SG_DEBUG("%d (%d) vs. %d : tInt8(%d),tUInt8(%d),tUInt32(%d),tInt32(%d)\n", typeno, PyArray_NOTYPE, NA_NumarrayType(arg), tInt8, tUInt8,tUInt32,tInt32);
 	switch (typeno)
 	{
 		case tInt8:
 		case tUInt8:
 			py_afeat  = NA_InputArray(arg, tUInt8, NUM_C_ARRAY);
 			if (NA_NDArrayCheck(arg))
-            {
-                if ((py_afeat->nd == 1))
+
+			{
+				if ((py_afeat->nd == 1))
 				{
 					CHAR* feat= (CHAR*) NA_OFFSETDATA(py_afeat);
 					int num_vec=py_afeat->dimensions[0];
 					int num_feat=0;
 					//int num_feat=PyArray(py_afeat)->itemsize;
 					SG_DEBUG( "vec: %d dim:%d\n", num_vec, num_feat);
-
 					if (feat)
 					{
 						if (args)
@@ -895,44 +908,49 @@ CFeatures* CGUIPython::set_features(PyObject* arg, char* args)
 							for(int i=0; i<num_vec; i++)
 							{
 								for(int j=0; j<num_feat; j++)
-									fm[i*num_feat+j]=feat[i*num_feat+j];
+								{
+									fm[j*num_vec+i]=feat[i*num_feat+j];
+								}
+
 							}
-							((CCharFeatures*) features)->set_feature_matrix(fm, num_feat, num_vec);
+							((CCharFeatures*) features)->set_feature_matrix(fm, num_vec, num_feat);
 						}
 						else
 							SG_ERROR( "please specify alphabet!\n");
 					}
-                    else
-                        SG_ERROR("empty feats ??\n");
-                }
-            }
+					else
+						SG_ERROR("empty feats ??\n");
+				}
+			}
 			else
-            {
-                if ((py_afeat->nd == 2))
-                {
-                    CHAR* feat= (CHAR*) NA_OFFSETDATA(py_afeat);
-                    int num_vec=py_afeat->dimensions[0];
-                    int num_feat=py_afeat->dimensions[1];
-
-                    if (feat)
-                    {
+			{
+				if ((py_afeat->nd == 2))
+				{
+					CHAR* feat= (CHAR*) NA_OFFSETDATA(py_afeat);
+					int num_vec=py_afeat->dimensions[0];
+					int num_feat=py_afeat->dimensions[1];
+					if (feat)
+					{
 						CAlphabet* alpha = new CAlphabet(args, strlen(args));
-                        features= new CCharFeatures(alpha, 0);
-                        CHAR* fm=new CHAR[num_vec*num_feat];
-                        ASSERT(fm);
-                        for(int i=0; i<num_vec; i++)
-                        {
-                            for(int j=0; j<num_feat; j++)
-                                fm[i*num_feat+j]=feat[i*num_feat+j];
-                        }
-                        ((CCharFeatures*) features)->set_feature_matrix(fm, num_feat, num_vec);
-                    }
-                    else
-                        SG_ERROR("empty feats ??\n");
-                }
-                else
-                    SG_ERROR( "set_features: arrays must have 2 dimension.\n");
-            }
+						features= new CCharFeatures(alpha, 0);
+						CHAR* fm=new CHAR[num_vec*num_feat];
+						ASSERT(fm);
+						for(int i=0; i<num_vec; i++)
+						{
+							for(int j=0; j<num_feat; j++)
+							{
+								fm[j*num_vec+i]=feat[i*num_feat+j];
+							}
+						}
+						((CCharFeatures*) features)->set_feature_matrix(fm, num_vec, num_feat);
+
+					}
+					else
+						SG_ERROR("empty feats ??\n");
+				}
+				else
+					SG_ERROR( "set_features: arrays must have 2 dimension.\n");
+			}
 			break;
 		case tFloat64:
 			py_afeat  = NA_InputArray(arg, tFloat64, NUM_C_ARRAY);
@@ -941,18 +959,22 @@ CFeatures* CGUIPython::set_features(PyObject* arg, char* args)
 				double* feat= (double*) NA_OFFSETDATA(py_afeat);
 				int num_vec=py_afeat->dimensions[0];
 				int num_feat=py_afeat->dimensions[1];
-
 				if (feat)
 				{
 					features= new CRealFeatures(0);
 					DREAL* fm=new DREAL[num_vec*num_feat];
 					ASSERT(fm);
+
 					for(int i=0; i<num_vec; i++)
 					{
 						for(int j=0; j<num_feat; j++)
-							fm[i*num_feat+j]=feat[i*num_feat+j];
+						{
+							fm[j*num_vec+i]=feat[i*num_feat+j];
+						}
+
 					}
-					((CRealFeatures*) features)->set_feature_matrix(fm, num_feat, num_vec);
+					((CRealFeatures*) features)->set_feature_matrix(fm, num_vec, num_feat);
+
 				}
 				else
 					SG_ERROR("empty feats ??\n");
@@ -962,7 +984,7 @@ CFeatures* CGUIPython::set_features(PyObject* arg, char* args)
 			break;
 		default:
 			SG_ERROR( "Unknown numpy type\n");
-	};
+	}
 
 	Py_XDECREF(py_afeat);
 	return features;
