@@ -27,8 +27,8 @@
 #include "distributions/hmm/HMM.h"
 #include "features/Labels.h"
 #include "features/RealFeatures.h"
-#include "kernel/WeightedDegreeCharKernel.h"
-#include "kernel/WeightedDegreePositionCharKernel.h"
+#include "kernel/WeightedDegreeStringKernel.h"
+#include "kernel/WeightedDegreePositionStringKernel.h"
 #include "kernel/CombinedKernel.h"
 #include "kernel/CommWordStringKernel.h"
 #include "kernel/CustomKernel.h"
@@ -381,7 +381,7 @@ extern "C"
 
 	SEXP CGUI_R::get_svm()
 	{
-		CSVM* svm=gui->guisvm.get_svm();
+		CSVM* svm=(CSVM*) gui->guiclassifier.get_classifier();
 
 		if (svm)
 		{
@@ -421,8 +421,7 @@ extern "C"
 		return(R_NilValue);
 	}
 
-
-	SEXP CGUI_R::svm_classify()
+	SEXP CGUI_R::classify()
 	{
 		CFeatures* f=gui->guifeatures.get_test_features();
 		if (f)
@@ -431,11 +430,11 @@ extern "C"
 			int num_vec=f->get_num_vectors();
 			PROTECT( output = allocMatrix(REALSXP, 1, num_vec) );
 
-			CLabels* l=gui->guisvm.classify();
+			CLabels* l=gui->guiclassifier.classify();
 
 			if (!l)
 			{
-				SG_ERROR( "svm_classify failed\n") ;
+				SG_ERROR( "classify failed\n") ;
 				return false ;
 			} ;
 
@@ -450,7 +449,6 @@ extern "C"
 		return R_NilValue;
 	}
 
-
 	bool CGUI_R::set_svm(SEXP arg_list)
 	{
 		if( TYPEOF(arg_list) != LISTSXP ) {
@@ -458,7 +456,7 @@ extern "C"
 			return false;
 		}
 
-		CSVM* svm=gui->guisvm.get_svm();
+		CSVM* svm=(CSVM*) gui->guiclassifier.get_classifier();
 
 		if (svm)
 		{
@@ -491,20 +489,19 @@ extern "C"
 		return false;
 	}
 
-	SEXP CGUI_R::svm_classify_example(int idx)
+	SEXP CGUI_R::classify_example(int idx)
 	{
-		/*
-		mxArray* mx_result=mxCreateDoubleMatrix(1, 1, mxDREAL);
-		retvals[0]=mx_result;
-		double* result=mxGetPr(mx_result);
-
-		if (!gui->guisvm.classify_example(idx, result[0]))
+		SEXP ans;
+		PROTECT( ans = allocVector(REALSXP, 1));
+ 
+		if (!gui->guiclassifier.classify_example(idx, REAL(ans)[0]))
 		{
+			REAL(ans)[0]=0;
 			SG_ERROR( "svm_classify_example failed\n") ;
-			return false ;
-		} ;
-*/
-		return R_NilValue;
+		}
+
+		UNPROTECT(1);
+		return ans;
 	}
 
 	SEXP CGUI_R::get_features(CFeatures* f)
@@ -744,7 +741,7 @@ bool CGUI_R::set_custom_kernel(SEXP args) {
 					if (alphabet && TYPEOF(alphabet) == STRSXP)
 					{
 						int num_vec = length(feat);
-						CHAR* al=CHAR(VECTOR_ELT(alphabet, 0));
+						CHAR* al=(CHAR*) CHAR(VECTOR_ELT(alphabet, 0));
 						CAlphabet* alpha = new CAlphabet(al, strlen(al));
 						T_STRING<CHAR>* sc=new T_STRING<CHAR>[num_vec];
 						ASSERT(alpha);
@@ -756,7 +753,7 @@ bool CGUI_R::set_custom_kernel(SEXP args) {
 						for (int i=0; i<num_vec; i++)
 						{
 							SEXPREC* s= STRING_ELT(feat,i);
-							CHAR* c= CHAR(s);
+							CHAR* c= (CHAR*) CHAR(s);
 							int len=LENGTH(s);
 
 							if (len && c)
@@ -811,7 +808,7 @@ SEXP CGUI_R::get_svm_objective() {
    SEXP ans;
 	//mxArray* mx_v=mxCreateDoubleMatrix(1, 1, mxDREAL);
 	PROTECT( ans = allocVector(REALSXP, 1));
-	CSVM* svm=gui->guisvm.get_svm();
+	CSVM* svm=(CSVM*) gui->guiclassifier.get_classifier();
 
 	if (svm)
 	{
@@ -884,28 +881,15 @@ SEXP CGUI_R::get_svm_objective() {
 			int num_vec1=k->get_lhs()->get_num_vectors();
 			int num_vec2=k->get_rhs()->get_num_vectors();
 			SG_DEBUG("Kernel matrix has size %d / %d\n", num_vec1, num_vec2);
-			SEXP ans,dim;
+			SEXP ans;
 			PROTECT( ans = allocMatrix(REALSXP, num_vec1, num_vec2) );
 
-			DREAL* blub = k->get_kernel_matrix_real(num_vec1, num_vec2,NULL);
-
-			if( blub == NULL )
-				SG_DEBUG("error return value is NULL!");
-
-			for( int i=0; i<num_vec1; i++) 
+			for (INT i=0; i<num_vec2; i++)
 			{
-				for( int j=0; j<num_vec2; j++) 
-				{
-					REAL(ans)[i+num_vec2*j] = blub[i+num_vec2*j];
-				}
+				for (INT j=0; j<num_vec1; j++)
+					REAL(ans)[i*num_vec1+j]=k->kernel(j,i);
 			}
-
-			PROTECT( dim = allocVector(INTSXP, 2) );
-			INTEGER(dim)[0] = num_vec1;
-			INTEGER(dim)[1] = num_vec2;
-			setAttrib(ans, R_DimSymbol, dim);
-
-			UNPROTECT(2);
+			UNPROTECT(1);
 			SG_DEBUG("matrix created!");
 			return ans;
 		}
@@ -926,7 +910,7 @@ SEXP CGUI_R::get_svm_objective() {
 			{
 				case K_WEIGHTEDDEGREE:
 					{
-						CWeightedDegreeCharKernel *kernel = (CWeightedDegreeCharKernel *) kernel_ ;
+						CWeightedDegreeStringKernel *kernel = (CWeightedDegreeStringKernel *) kernel_ ;
 
 						if (kernel->get_max_mismatch()!=0)
 							return false ;
@@ -945,7 +929,7 @@ SEXP CGUI_R::get_svm_objective() {
 					}
 				case K_WEIGHTEDDEGREEPOS:
 					{
-						CWeightedDegreePositionCharKernel *kernel = (CWeightedDegreePositionCharKernel *) kernel_ ;
+						CWeightedDegreePositionStringKernel *kernel = (CWeightedDegreePositionStringKernel *) kernel_ ;
 
 						if (kernel->get_max_mismatch()!=0)
 							return false ;
@@ -1006,7 +990,7 @@ SEXP CGUI_R::get_svm_objective() {
 
 		if (kernel_ && (kernel_->get_kernel_type() == K_WEIGHTEDDEGREE))
 		{
-			CWeightedDegreeCharKernel *kernel = (CWeightedDegreeCharKernel *) kernel_ ;
+			CWeightedDegreeStringKernel *kernel = (CWeightedDegreeStringKernel *) kernel_ ;
 
 			if (!kernel->is_tree_initialized())
 			{
@@ -1041,7 +1025,7 @@ SEXP CGUI_R::get_svm_objective() {
 		}
 		if (kernel_ && (kernel_->get_kernel_type() == K_WEIGHTEDDEGREEPOS))
 		{
-			CWeightedDegreePositionCharKernel *kernel = (CWeightedDegreePositionCharKernel *) kernel_ ;
+			CWeightedDegreePositionStringKernel *kernel = (CWeightedDegreePositionStringKernel *) kernel_ ;
 
 			if (!kernel->is_tree_initialized())
 			{
@@ -1097,7 +1081,7 @@ SEXP CGUI_R::get_svm_objective() {
 
 		if (kernel_ && (kernel_->get_kernel_type() == K_WEIGHTEDDEGREE))
 		{
-			CWeightedDegreeCharKernel *kernel = (CWeightedDegreeCharKernel *) kernel_ ;
+			CWeightedDegreeStringKernel *kernel = (CWeightedDegreeStringKernel *) kernel_ ;
 
 			//SG_DEBUG("getting degree weights...");
 			const DREAL* weights = kernel->get_degree_weights(degree, length) ;
@@ -1116,7 +1100,7 @@ SEXP CGUI_R::get_svm_objective() {
 
 		if (kernel_ && (kernel_->get_kernel_type() == K_WEIGHTEDDEGREEPOS))
 		{
-			CWeightedDegreePositionCharKernel *kernel = (CWeightedDegreePositionCharKernel *) kernel_ ;
+			CWeightedDegreePositionStringKernel *kernel = (CWeightedDegreePositionStringKernel *) kernel_ ;
 
 			const DREAL* weights = kernel->get_degree_weights(degree, length) ;
 			if (length == 0)
@@ -1166,7 +1150,7 @@ SEXP CGUI_R::get_svm_objective() {
 
 			if (kernel_ && (kernel_->get_kernel_type() == K_WEIGHTEDDEGREE))
 			{
-				CWeightedDegreeCharKernel *kernel = (CWeightedDegreeCharKernel *) kernel_ ;
+				CWeightedDegreeStringKernel *kernel = (CWeightedDegreeStringKernel *) kernel_ ;
 
 				const DREAL* weights = kernel->get_degree_weights(degree, length) ;
 				if (length == 0)
@@ -1184,7 +1168,7 @@ SEXP CGUI_R::get_svm_objective() {
 
 			if (kernel_ && (kernel_->get_kernel_type() == K_WEIGHTEDDEGREEPOS))
 			{
-				CWeightedDegreePositionCharKernel *kernel = (CWeightedDegreePositionCharKernel *) kernel_ ;
+				CWeightedDegreePositionStringKernel *kernel = (CWeightedDegreePositionStringKernel *) kernel_ ;
 
 				const DREAL* weights = kernel->get_degree_weights(degree, length) ;
 				if (length == 0)
@@ -1230,7 +1214,7 @@ SEXP CGUI_R::get_svm_objective() {
 
 		if (kernel_ && (kernel_->get_kernel_type() == K_WEIGHTEDDEGREE))
 		{
-			CWeightedDegreeCharKernel *kernel = (CWeightedDegreeCharKernel *) kernel_ ;
+			CWeightedDegreeStringKernel *kernel = (CWeightedDegreeStringKernel *) kernel_ ;
 
 			const DREAL* position_weights = kernel->get_position_weights(length) ;
 			mxArray* mx_result ;
@@ -1249,7 +1233,7 @@ SEXP CGUI_R::get_svm_objective() {
 		}
 		if (kernel_ && (kernel_->get_kernel_type() == K_WEIGHTEDDEGREEPOS))
 		{
-			CWeightedDegreePositionCharKernel *kernel = (CWeightedDegreePositionCharKernel *) kernel_ ;
+			CWeightedDegreePositionStringKernel *kernel = (CWeightedDegreePositionStringKernel *) kernel_ ;
 
 			const DREAL* position_weights = kernel->get_position_weights(length) ;
 			mxArray* mx_result ;
@@ -1275,7 +1259,7 @@ SEXP CGUI_R::get_svm_objective() {
 
 		if (kernel_ && (kernel_->get_kernel_type() == K_WEIGHTEDDEGREE))
 		{
-			CWeightedDegreeCharKernel *kernel = (CWeightedDegreeCharKernel *) kernel_ ;
+			CWeightedDegreeStringKernel *kernel = (CWeightedDegreeStringKernel *) kernel_ ;
 			INT degree = kernel->get_degree() ;
 			if (mxGetM(mx_arg)!=degree || mxGetN(mx_arg)<1)
 			{
@@ -1294,7 +1278,7 @@ SEXP CGUI_R::get_svm_objective() {
 
 		if (kernel_ && (kernel_->get_kernel_type() == K_WEIGHTEDDEGREEPOS))
 		{
-			CWeightedDegreePositionCharKernel *kernel = (CWeightedDegreePositionCharKernel *) kernel_ ;
+			CWeightedDegreePositionStringKernel *kernel = (CWeightedDegreePositionStringKernel *) kernel_ ;
 			INT degree = kernel->get_degree() ;
 			if (mxGetM(mx_arg)!=degree || mxGetN(mx_arg)<1)
 			{
@@ -1332,7 +1316,7 @@ SEXP CGUI_R::get_svm_objective() {
 
 			if (kernel_ && (kernel_->get_kernel_type() == K_WEIGHTEDDEGREE))
 			{
-				CWeightedDegreeCharKernel *kernel = (CWeightedDegreeCharKernel *) kernel_ ;
+				CWeightedDegreeStringKernel *kernel = (CWeightedDegreeStringKernel *) kernel_ ;
 				INT degree = kernel->get_degree() ;
 				if (mxGetM(mx_arg)!=degree || mxGetN(mx_arg)<1)
 				{
@@ -1350,7 +1334,7 @@ SEXP CGUI_R::get_svm_objective() {
 			}
 			if (kernel_ && (kernel_->get_kernel_type() == K_WEIGHTEDDEGREEPOS))
 			{
-				CWeightedDegreePositionCharKernel *kernel = (CWeightedDegreePositionCharKernel *) kernel_ ;
+				CWeightedDegreePositionStringKernel *kernel = (CWeightedDegreePositionStringKernel *) kernel_ ;
 				INT degree = kernel->get_degree() ;
 				if (mxGetM(mx_arg)!=degree || mxGetN(mx_arg)<1)
 				{
@@ -1393,7 +1377,7 @@ SEXP CGUI_R::get_svm_objective() {
 
 		if (kernel_ && (kernel_->get_kernel_type() == K_WEIGHTEDDEGREE))
 		{
-			CWeightedDegreeCharKernel *kernel = (CWeightedDegreeCharKernel *) kernel_ ;
+			CWeightedDegreeStringKernel *kernel = (CWeightedDegreeStringKernel *) kernel_ ;
 			if (mxGetM(mx_arg)!=1 & mxGetN(mx_arg)>0)
 			{
 				SG_ERROR( "dimension mismatch (should be 1xseq_length or 0x0)\n") ;
@@ -1405,7 +1389,7 @@ SEXP CGUI_R::get_svm_objective() {
 		}
 		if (kernel_ && (kernel_->get_kernel_type() == K_WEIGHTEDDEGREEPOS))
 		{
-			CWeightedDegreePositionCharKernel *kernel = (CWeightedDegreePositionCharKernel *) kernel_ ;
+			CWeightedDegreePositionStringKernel *kernel = (CWeightedDegreePositionStringKernel *) kernel_ ;
 			if (mxGetM(mx_arg)!=1 & mxGetN(mx_arg)>0)
 			{
 				SG_ERROR( "dimension mismatch (should be 1xseq_length or 0x0)\n") ;
