@@ -20,15 +20,21 @@ extern "C" {
 #include "kernel/DiagKernel.h"
 #include "lib/io.h"
 
-CDiagKernel::CDiagKernel(LONG size, DREAL d)
-  : CKernel(size),diag(d)
+CDiagKernel::CDiagKernel(INT size, DREAL d)
+	: CKernel(size), diag(d)
 {
 }
 
-CDiagKernel::~CDiagKernel() 
+CDiagKernel::CDiagKernel(CFeatures* l, CFeatures* r, DREAL d)
+	: CKernel(10), diag(d)
+{
+	init(l, r);
+}
+
+CDiagKernel::~CDiagKernel()
 {
 }
-  
+
 void CDiagKernel::cleanup()
 {
 }
