@@ -13,12 +13,20 @@
 #include "lib/common.h"
 #include "lib/io.h"
 #include "kernel/DistanceKernel.h"
+#include "features/RealFeatures.h"
 
-CDistanceKernel::CDistanceKernel(LONG cache,DREAL w ,CDistance* dist)
-  :CKernel(cache),distance(dist),width(w)
+CDistanceKernel::CDistanceKernel(INT size, DREAL w ,CDistance* d)
+	: CKernel(size), distance(d), width(w)
 {
 }
 
+CDistanceKernel::CDistanceKernel(
+	CFeatures *l, CFeatures *r, DREAL w , CDistance* d)
+	: CKernel(10), distance(d), width(w)
+{
+	init(l, r);
+}
+
 CDistanceKernel::~CDistanceKernel()
 {
     cleanup();
@@ -27,9 +35,7 @@ CDistanceKernel::~CDistanceKernel()
 
 bool CDistanceKernel::init(CFeatures* l, CFeatures* r)
 {
-    bool result0=CKernel::init(l,r);
-    bool result1=distance->init(l,r);
-    return result0&&result1;
+    return CKernel::init(l,r) && distance->init(l,r);
 }
 
 void CDistanceKernel::cleanup()
