@@ -10,12 +10,15 @@
 
 #include "kernel/KernelMachine.h"
 
-CKernelMachine::CKernelMachine():CClassifier(),kernel(NULL)
+CKernelMachine::CKernelMachine()
+	: CClassifier(), kernel(NULL), use_batch_computation(true),
+	use_linadd(true)
 {
 }
 
 CKernelMachine::~CKernelMachine()
 {
+    SG_REF(kernel);
 }
 
 CLabels* CKernelMachine::classify(CLabels* output)
@@ -28,7 +31,7 @@ CLabels* CKernelMachine::classify(CLabels* output)
 		if (!output)
 			output=new CLabels(num);
 
-		ASSERT(output);
+		ASSERT(output && output->get_num_labels() == num);
 		for (INT i=0; i<num; i++)
 			output->set_label(i, classify_example(i));
 
