@@ -6,6 +6,7 @@
  *
  * Written (W) 1999-2007 Soeren Sonnenburg
  * Written (W) 1999-2007 Gunnar Raetsch
+ * Written (W) 2007 Konrad Rieck
  * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
@@ -15,10 +16,12 @@
 #include "lib/common.h"
 #include "lib/io.h"
 #include "base/SGObject.h"
+#include "base/Parallel.h"
 
 #include <math.h>
 #include <stdio.h>
 #include <float.h>
+#include <pthread.h>
 
 #ifdef HAVE_LAPACK
 extern "C" {
@@ -58,6 +61,37 @@ extern "C" {
 extern "C" int	finite(double);
 #endif
 
+/* Maximum stack size */
+#define RADIX_STACK_SIZE	    512
+
+/* Stack macros */
+#define radix_push(a, n, i)	    sp->sa = a, sp->sn = n, (sp++)->si = i
+#define radix_pop(a, n, i)	    a = (--sp)->sa, n = sp->sn, i = sp->si
+
+/* Stack structure */
+template <class T> struct radix_stack_t
+{
+    T *sa;				/* Pointer to pile */
+    size_t sn;			/* Number of grams in pile */
+    unsigned short si;	/* Byte in current focus */
+};
+
+struct pair
+{
+	int idx1,idx2;
+};
+
+struct thread_qsort
+{
+	DREAL* output;
+	INT* index;
+	INT size;
+
+	INT* qsort_threads;
+	INT sort_limit;
+	INT num_threads;
+};
+
 /** Mathematical Functions.
  * Class which collects generic mathematical functions
  */
@@ -93,6 +127,18 @@ public:
 		return ((a)>=(b))?(a):(b);
 	}
 
+	///return the value clamped to interval [lb,ub]
+	template <class T>
+	static inline T clamp(T value, T lb, T ub) 
+	{
+		if (value<=lb)
+			return lb;
+		else if (value>=ub)
+			return ub;
+		else
+			return value;
+	}
+
 	///return the maximum of two integers
 	template <class T>
 	static inline T abs(T a)
@@ -131,12 +177,13 @@ public:
 		else return (a<0) ? (-1) : (+1);
 	}
 
-	/// swap floats a and b
+	/// swap e.g. floats a and b
 	template <class T>
 	static inline void swap(T & a,T &b)
 	{
-		T c=a ;
-		a=b; b=c ;
+		T c=a;
+		a=b;
+		b=c;
 	}
 
 	/// x^2
@@ -157,6 +204,16 @@ public:
 		return ::pow(x, n);
 #endif
 	}
+	
+	static inline INT pow(INT x, INT n)
+	{
+		ASSERT(n>=0);
+		INT result=1;
+		while (n--)
+			result*=x;
+
+		return result;
+	}
 
 	static inline DREAL pow(DREAL x, DREAL n)
 	{
@@ -168,6 +225,18 @@ public:
 		return ::log(v)/::log(10.0);
 	}
 
+#ifdef HAVE_LOG2
+	static inline DREAL log2(DREAL v)
+	{
+		return ::log2(v);
+	}
+#else
+	static inline DREAL log2(DREAL v)
+	{
+		return ::log(v)/::log(2.0);
+	}
+#endif //HAVE_LOG2
+
 	static inline DREAL log(DREAL v)
 	{
 		return ::log(v);
@@ -177,6 +246,26 @@ public:
 	/// return the pseudo inverse for matrix
 	/// when matrix has shape (rows, cols) the pseudo inverse has (cols, rows)
 	static DREAL* pinv(DREAL* matrix, INT rows, INT cols, DREAL* target=NULL);
+
+
+	//C := alpha*op( A )*op( B ) + beta*C
+	//op( X ) = X   or   op( X ) = X',
+	static inline void dgemm(double alpha, const double* A, int rows, int cols, CBLAS_TRANSPOSE transposeA,
+			double *B, int cols_B, CBLAS_TRANSPOSE transposeB,
+			double beta, double *C)
+	{
+		cblas_dgemm(CblasColMajor, transposeA, transposeB, rows, cols, cols_B,
+				alpha, A, cols, B, cols_B, beta, C, cols);
+	}
+
+	//y := alpha*A*x + beta*y,   or   y := alpha*A'*x + beta*y,
+	static inline void dgemv(double alpha, const double *A, int rows, int cols, const enum CBLAS_TRANSPOSE transposeA,
+                 const double* X, double beta, double* Y)
+	{
+		cblas_dgemv(CblasColMajor, transposeA,
+				rows, cols, alpha, A, cols,
+				X, 1, beta, Y, 1);
+	}
 #endif
 
 	static inline LONG factorial(INT n)
@@ -187,9 +276,9 @@ public:
 		return res ;
 	}
 	
-	static INT random()
+	static LONG random()
 	{
-#ifdef CYGWIN
+#if defined(CYGWIN) || defined(__INTERIX)
 		return rand();
 #else
 		return ::random();
@@ -198,20 +287,38 @@ public:
 
 	static INT random(INT min_value, INT max_value)
 	{
-		DREAL ret = min_value + (max_value+1-min_value)*(1.*rand())/(RAND_MAX*1.) ;
-		INT iret = (INT) ::floor(ret - 1e-10) ;
-		ASSERT(iret >= min_value && iret<=max_value) ;
-		return iret ;
+		INT ret = min_value + (INT) ((max_value-min_value+1) * (random() / (RAND_MAX+1.0)));
+		ASSERT(ret >= min_value && ret<=max_value);
+		return ret ;
 	}
 
 	static DREAL random(DREAL min_value, DREAL max_value)
 	{
-		DREAL ret = min_value + (max_value-min_value)*(1.*rand())/(RAND_MAX*1.) ;
-		if (!(ret >= min_value-1e-6 && ret<=max_value+1e-6))
-			fprintf(stderr, "%f %f %f\n", ret, min_value, max_value) ;
-		
-		ASSERT(ret >= min_value-1e-6 && ret<=max_value+1e-6) ;
-		return ret ;
+		DREAL ret = min_value + ((max_value-min_value) * (random() / (1.0*RAND_MAX)));
+
+		if (ret<min_value || ret>max_value)
+			SG_SPRINT("min_value:%10.10f value: %10.10f max_value:%10.10f", min_value, ret, max_value);
+		ASSERT(ret >= min_value && ret<=max_value);
+		return ret;
+	}
+
+    template <class T>
+	static void fill_vector(T* vec, INT len, T value)
+	{
+		for (INT i=0; i<len; i++)
+			vec[i]=value;
+	}
+    template <class T>
+	static void range_fill_vector(T* vec, INT len, T start=0)
+	{
+		for (INT i=0; i<len; i++)
+			vec[i]=i+start;
+	}
+
+	static void random_vector(DREAL* vec, INT len, DREAL min_value, DREAL max_value)
+	{
+		for (INT i=0; i<len; i++)
+			vec[i]=CMath::random(min_value, max_value);
 	}
 
 	static inline INT* randperm(INT n)
@@ -237,7 +344,21 @@ public:
 		return res/factorial(k) ;
 	}
 
-	static inline DREAL dot(DREAL* v1, DREAL* v2, INT n)
+	static inline void scalar_times_vec(DREAL scalar, DREAL* vec, INT n)
+	{
+		for (INT i=0; i<n; i++)
+			vec[i]*=scalar;
+	}
+
+	/// x=x+alpha*y
+	static inline void vec1_plus_scalar_times_vec2(DREAL* vec1,
+			DREAL scalar, const DREAL* vec2, INT n)
+	{
+		for (INT i=0; i<n; i++)
+			vec1[i]+=scalar*vec2[i];
+	}
+
+	static inline DREAL dot(const DREAL* v1, const DREAL* v2, INT n)
 	{
 		DREAL r=0;
 #ifdef HAVE_LAPACK
@@ -250,6 +371,33 @@ public:
 		return r;
 	}
 
+	/// target=alpha*vec1 + beta*vec2
+	static inline void add(DREAL* target, DREAL alpha, const DREAL* v1, DREAL beta, const DREAL* v2, INT len)
+	{
+		for (INT i=0; i<len; i++)
+			target[i]=alpha*v1[i]+beta*v2[i];
+	}
+
+	/// return sum(vec)
+	static inline DREAL sum(DREAL* vec, INT len)
+	{
+		DREAL result=0;
+		for (INT i=0; i<len; i++)
+			result+=vec[i];
+
+		return result;
+	}
+
+	/// return sum(abs(vec))
+	static inline DREAL sum_abs(DREAL* vec, INT len)
+	{
+		DREAL result=0;
+		for (INT i=0; i<len; i++)
+			result+=CMath::abs(vec[i]);
+
+		return result;
+	}
+
 	static inline DREAL mean(DREAL* vec, INT len)
 	{
 		ASSERT(vec);
@@ -270,11 +418,147 @@ public:
 	}
 
 	/** performs a bubblesort on a given matrix a.
-	 * it is sorted from in ascending order from top to bottom
+	 * it is sorted in ascending order from top to bottom
 	 * and left to right */
 	static void sort(INT *a, INT cols, INT sort_col=0) ;
 	static void sort(DREAL *a, INT*idx, INT N) ;
-	
+
+	/*
+	 * Inline function to extract the byte at position p (from left)
+	 * of an 64 bit integer. The function is somewhat identical to 
+	 * accessing an array of characters via [].
+	 */
+
+	/** performs a in-place radix sort in ascending order */
+	template <class T>
+	inline static void radix_sort(T* array, size_t size)
+	{
+		radix_sort_helper(array,size,0);
+	}
+
+	template <class T>
+	static inline BYTE byte(T word, unsigned short p)
+	{
+		return (word >> (sizeof(T)-p-1) * 8) & 0xff;
+	}
+
+	template <class T>
+	static void radix_sort_helper(T* array, size_t size, unsigned short i)
+	{
+		static size_t count[256], nc, cmin;
+		T *ak;
+		BYTE c=0;
+		radix_stack_t<T> s[RADIX_STACK_SIZE], *sp, *olds, *bigs;
+		T *an, *aj, *pile[256];
+		size_t *cp, cmax;
+
+		/* Push initial array to stack */
+		sp = s;
+		radix_push(array, size, i);
+
+		/* Loop until all digits have been sorted */
+		while (sp>s) {
+			radix_pop(array, size, i);
+			an = array + size;
+
+			/* Make character histogram */
+			if (nc == 0) {
+				cmin = 0xff;
+				for (ak = array; ak < an; ak++) {
+					c = byte(*ak, i);
+					count[c]++;
+					if (count[c] == 1) {
+						/* Determine smallest character */
+						if (c < cmin)
+							cmin = c;
+						nc++;
+					}
+				}
+
+				/* Sort recursively if stack size too small */
+				if (sp + nc > s + RADIX_STACK_SIZE) {
+					radix_sort_helper(array, size, i);
+					continue;
+				}
+			}
+
+			/*
+			 * Set pile[]; push incompletely sorted bins onto stack.
+			 * pile[] = pointers to last out-of-place element in bins.
+			 * Before permuting: pile[c-1] + count[c] = pile[c];
+			 * during deal: pile[c] counts down to pile[c-1].
+			 */
+			olds = bigs = sp;
+			cmax = 2;
+			ak = array;
+			pile[0xff] = an;
+			for (cp = count + cmin; nc > 0; cp++) {
+				/* Find next non-empty pile */
+				while (*cp == 0)
+					cp++;
+				/* Pile with several entries */
+				if (*cp > 1) {
+					/* Determine biggest pile */
+					if (*cp > cmax) {
+						cmax = *cp;
+						bigs = sp;
+					}
+
+					if (i < sizeof(T)-1)
+						radix_push(ak, *cp, (unsigned short) (i + 1));
+				}
+				pile[cp - count] = ak += *cp;
+				nc--;
+			}
+
+			/* Play it safe -- biggest bin last. */
+			swap(*olds, *bigs);
+
+			/*
+			 * Permute misplacements home. Already home: everything
+			 * before aj, and in pile[c], items from pile[c] on.
+			 * Inner loop:
+			 *      r = next element to put in place;
+			 *      ak = pile[r[i]] = location to put the next element.
+			 *      aj = bottom of 1st disordered bin.
+			 * Outer loop:
+			 *      Once the 1st disordered bin is done, ie. aj >= ak,
+			 *      aj<-aj + count[c] connects the bins in array linked list;
+			 *      reset count[c].
+			 */
+			aj = array;
+			while (aj<an)
+			{
+				T r;
+
+				for (r = *aj; aj < (ak = --pile[c = byte(r, i)]);)
+					swap(*ak, r);
+
+				*aj = r;
+				aj += count[c];
+				count[c] = 0;
+			}
+		}
+	}
+	/** performs insertion sort of an array output of length size
+	 * it is sorted from in ascending (for type T) */
+	template <class T>
+	static void insertion_sort(T* output, INT size)
+	{
+		for (INT i=0; i<size-1; i++)
+		{
+			INT j=i-1;
+			T value=output[i];
+			while (j >= 0 && output[j] > value)
+			{
+				output[j+1] = output[j];
+				j--;
+			}
+			output[j+1]=value;
+		}
+	}
+	 
+
 	/** performs a quicksort on an array output of length size
 	 * it is sorted from in ascending (for type T) */
 	//template <class T>
@@ -322,20 +606,23 @@ public:
 	template <class T> static void display_matrix(T* matrix, INT rows, INT cols, const char* name="matrix");
 
 	/** performs a quicksort on an array output of length size
-	 * it is sorted from in ascending order 
+	 * it is sorted in ascending order 
 	 * (for type T1) and returns the index (type T2)
 	 * matlab alike [sorted,index]=sort(output) 
 	 */
 	template <class T1,class T2>
-	static void qsort(T1* output, T2* index, INT size);
+	static void qsort_index(T1* output, T2* index, UINT size);
+
+	template <class T1,class T2>
+	static void* parallel_qsort_index(void* p);
 
 	/** performs a quicksort on an array output of length size
-	 * it is sorted from in ascending order
+	 * it is sorted in ascending order
 	 * (for type T1) and returns the index (type T2)
 	 * matlab alike [sorted,index]=sort(output) 
 	 */
 	template <class T1,class T2>
-	static void qsort_backward(T1* output, T2* index, INT size);
+	static void qsort_backward_index(T1* output, T2* index, INT size);
 
      /* finds the smallest element in output and puts that element as the 
 		first element  */
@@ -416,6 +703,10 @@ public:
 		return -1;
 	}
 
+	/// align two sequences seq1 & seq2 of length l1 and l2 using gapCost
+	/// return alignment cost
+	static DREAL Align(CHAR * seq1, CHAR* seq2, INT l1, INT l2, DREAL gapCost);
+
 	/** calculates ROC into (fp,tp)
 	 * from output and label of length size 
 	 * returns index with smallest error=fp+fn
@@ -524,6 +815,7 @@ public:
 	//@{
 	/// infinity
 	static const DREAL INFTY;
+	static const DREAL ALMOST_INFTY;
 	
 	/// almost neg (log) infinity
 	static const DREAL ALMOST_NEG_INFTY;
@@ -546,10 +838,108 @@ protected:
 	static CHAR rand_state[256];
 };
 
+template <class T1,class T2>
+void* CMath::parallel_qsort_index(void* p)
+{
+	struct thread_qsort* ps=(thread_qsort*) p;
+	T1* output=ps->output;
+	T2* index=ps->index;
+	INT size=ps->size;
+	INT* qsort_threads=ps->qsort_threads;
+	INT sort_limit=ps->sort_limit;
+	INT num_threads=ps->num_threads;
+
+	if (size==2)
+	{
+		if (output[0] > output [1])
+		{
+			swap(output[0], output[1]);
+			swap(index[0], index[1]);
+		}
+		return NULL;
+	}
+	/*double split=output[(((uint64_t) size)*rand())/(((uint64_t)RAND_MAX)+1)];*/
+	double split=output[size/2];
+
+	INT left=0;
+	INT right=size-1;
+
+	while (left<=right)
+	{
+		while (output[left] < split)
+			left++;
+		while (output[right] > split)
+			right--;
+
+		if (left<=right)
+		{
+			swap(output[left], output[right]);
+			swap(index[left], index[right]);
+			left++;
+			right--;
+		}
+	}
+	bool lthread_start=false;
+	bool rthread_start=false;
+	pthread_t lthread;
+	pthread_t rthread;
+	struct thread_qsort t1;
+	struct thread_qsort t2;
+
+	if (right+1> 1 && (right+1< sort_limit || *qsort_threads >= num_threads-1))
+		qsort_index(output,index,right+1);
+	else if (right+1> 1)
+	{
+		*qsort_threads++;
+		lthread_start=true;
+		t1.output=output;
+		t1.index=index;
+		t1.size=right+1;
+		if (pthread_create(&lthread, NULL, parallel_qsort_index<T1,T2>, (void*) &t1) != 0)
+		{
+			lthread_start=false;
+			*qsort_threads--;
+			qsort_index(output,index,right+1);
+		}
+	}
+
+
+	if (size-left> 1 && (size-left< sort_limit || *qsort_threads >= num_threads-1))
+		qsort_index(&output[left],&index[left], size-left);
+	else if (size-left> 1)
+	{
+		*qsort_threads++;
+		rthread_start=true;
+		t2.output=&output[left];
+		t2.index=&index[left];
+		t2.size=size-left;
+		if (pthread_create(&rthread, NULL, parallel_qsort_index<T1,T2>, (void*)&t2) != 0)
+		{
+			rthread_start=false;
+			*qsort_threads--;
+			qsort_index(&output[left],&index[left], size-left);
+		}
+	}
+
+	if (lthread_start)
+	{
+		pthread_join(lthread, NULL);
+		*qsort_threads--;
+	}
+
+	if (rthread_start)
+	{
+		pthread_join(rthread, NULL);
+		*qsort_threads--;
+	}
+
+	return NULL;
+}
+
 
 //implementations of template functions
 template <class T1,class T2>
-void CMath::qsort(T1* output, T2* index, INT size)
+void CMath::qsort_index(T1* output, T2* index, UINT size)
 {
 	if (size==2)
 	{
@@ -582,14 +972,14 @@ void CMath::qsort(T1* output, T2* index, INT size)
 	}
 
 	if (right+1> 1)
-		qsort(output,index,right+1);
+		qsort_index(output,index,right+1);
 
 	if (size-left> 1)
-		qsort(&output[left],&index[left], size-left);
+		qsort_index(&output[left],&index[left], size-left);
 }
 
 template <class T1,class T2>
-void CMath::qsort_backward(T1* output, T2* index, INT size)
+void CMath::qsort_backward_index(T1* output, T2* index, INT size)
 {
 	if (size==2)
 	{
@@ -623,10 +1013,10 @@ void CMath::qsort_backward(T1* output, T2* index, INT size)
 	}
 	
 	if (right+1> 1)
-		qsort(output,index,right+1);
+		qsort_backward_index(output,index,right+1);
 	
 	if (size-left> 1)
-		qsort(&output[left],&index[left], size-left);
+		qsort_backward_index(&output[left],&index[left], size-left);
 }
 
 template <class T> 
@@ -636,58 +1026,26 @@ void CMath::nmin(DREAL* output, T* index, INT size, INT n)
 		for (INT i=0; i<n; i++)
 			min(&output[i], &index[i], size-i) ;
 	else
-		qsort(output, index, size) ;
+		qsort_index(output, index, size) ;
 }
 
+/* move the smallest entry in the array to the beginning */
 template <class T>
 void CMath::min(DREAL* output, T* index, INT size)
 {
-	if (size<=0)
-		return ;
-	DREAL min_elem = output[0] ;
-	INT min_index = 0 ;
+	if (size<=1)
+		return;
+	DREAL min_elem=output[0];
+	INT min_index=0;
 	for (INT i=1; i<size; i++)
 	{
 		if (output[i]<min_elem)
 		{
-			min_index=i ;
-			min_elem=output[i] ;
+			min_index=i;
+			min_elem=output[i];
 		}
 	}
-	swap(output[0], output[min_index]) ;
-	swap(index[0], index[min_index]) ;
+	swap(output[0], output[min_index]);
+	swap(index[0], index[min_index]);
 }
-
-template <class T>
-void CMath::display_vector(T* vector, INT n, const char* name)
-{
-	SG_SPRINT("%s=[", name);
-	for (INT i=0; i<n-1; i++)
-		SG_SPRINT("%f,", (double) vector[i]);
-	for (INT i=n-1; i<n; i++)
-		SG_SPRINT("%f]\n", (double) vector[i]);
-}
-
-template <class T>
-void CMath::display_matrix(T* matrix, INT rows, INT cols, const char* name)
-{
-	SG_SPRINT("%s=[\n", name);
-	for (INT j=0; j<rows-1; j++)
-	{
-		SG_SPRINT("[");
-		for (INT i=0; i<cols-1; i++)
-			SG_SPRINT("\t%f,", (double) matrix[j+i*rows]);
-		for (INT i=cols-1; i<cols; i++)
-			SG_SPRINT("\t%f],\n", (double) matrix[j+i*rows]);
-	}
-	for (INT j=rows-1; j<rows; j++)
-	{
-		SG_SPRINT("[");
-		for (INT i=0; i<cols-1; i++)
-			SG_SPRINT("\t%f,", (double) matrix[j+i*rows]);
-		for (INT i=cols-1; i<cols; i++)
-			SG_SPRINT("\t%f]\n]\n", (double) matrix[j+i*rows]);
-	}
-}
-
 #endif
