@@ -18,6 +18,8 @@
 #include <signal.h>
 #define NUMTRAPPEDSIGS 2
 
+#include "lib/python.h"
+
 class CSignal : public CSGObject
 {
 public:
@@ -29,7 +31,27 @@ public:
 	static bool set_handler();
 	static bool unset_handler();
 	static void clear();
-	static inline bool cancel_computations() { return cancel_computation; }
+	static inline bool cancel_computations()
+	{
+#ifdef HAVE_PYTHON
+	  if (PyErr_CheckSignals())
+	  {
+		  SG_SPRINT("\nImmediately return to matlab prompt / Prematurely finish computations / Do nothing (I/P/D)? ");
+		  char answer=fgetc(stdin);
+
+		  if (answer == 'I')
+			  SG_SERROR("shogun stopped by SIGINT\n");
+		  else if (answer == 'P')
+		  {
+			  PyErr_Clear();
+			  cancel_computation=true;
+		  }
+		  else
+			  SG_SPRINT("\n");
+	  }
+#endif
+		return cancel_computation;
+	}
 protected:
 	static int signals[NUMTRAPPEDSIGS];
 	static struct sigaction oldsigaction[NUMTRAPPEDSIGS];
